import strawberry


@strawberry.type
class Query:
    abc: str


def test_enable_graphiql_view_and_allow_queries_via_get():
    from fastapi import FastAPI

    from strawberry.fastapi import GraphQLRouter

    app = FastAPI()
    schema = strawberry.Schema(query=Query)
    graphql_app = GraphQLRouter[None, None](schema)
    app.include_router(graphql_app, prefix="/graphql")

    assert "get" in app.openapi()["paths"]["/graphql"]
    assert "post" in app.openapi()["paths"]["/graphql"]


def test_disable_graphiql_view_and_allow_queries_via_get():
    from fastapi import FastAPI

    from strawberry.fastapi import GraphQLRouter

    app = FastAPI()
    schema = strawberry.Schema(query=Query)
    graphql_app = GraphQLRouter[None, None](
        schema, graphql_ide=None, allow_queries_via_get=False
    )
    app.include_router(graphql_app, prefix="/graphql")

    assert "get" not in app.openapi()["paths"]["/graphql"]
    assert "post" in app.openapi()["paths"]["/graphql"]


def test_graphql_router_with_tags():
    from fastapi import FastAPI

    from strawberry.fastapi import GraphQLRouter

    app = FastAPI()
    schema = strawberry.Schema(query=Query)
    graphql_app = GraphQLRouter[None, None](schema, tags=["abc"])
    app.include_router(graphql_app, prefix="/graphql")

    assert "abc" in app.openapi()["paths"]["/graphql"]["get"]["tags"]
    assert "abc" in app.openapi()["paths"]["/graphql"]["post"]["tags"]



"""The following tests ensure that the types are resolved using the correct
module. Concrete types should be non-problematic and are only included
here for completeness. A problematic case is when a type is a string
(forward reference) and can only be resolved at schema construction.
"""

import strawberry

from . import a_mod, b_mod, c_mod, x_mod


def test_a():
    @strawberry.type
    class Query:
        a_list: list[a_mod.AObject]

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[a_mod.AObject]


def test_a_resolver():
    @strawberry.type
    class Query:
        a_list: list[a_mod.AObject] = strawberry.field(resolver=a_mod.a_resolver)

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[a_mod.AObject]


def test_a_only_resolver():
    @strawberry.type
    class Query:
        a_list = strawberry.field(resolver=a_mod.a_resolver)

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[a_mod.AObject]


def test_a_typeless_resolver():
    @strawberry.type
    class Query:
        a_list: list[a_mod.AObject] = strawberry.field(resolver=x_mod.typeless_resolver)

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[a_mod.AObject]


def test_c_composition_by_name():
    [
        a_field,
        b_field,
        a_method,
        b_method,
    ] = c_mod.CCompositionByName.__strawberry_definition__.fields
    assert a_field.type == list[a_mod.AObject]
    assert b_field.type == list[b_mod.BObject]
    assert a_method.type == list[a_mod.AObject]
    assert b_method.type == list[b_mod.BObject]


def test_c_inheritance():
    [
        a_name,
        a_age,
        a_is_of_full_age,
        b_name,
        b_age,
        b_is_of_full_age,
    ] = c_mod.CInheritance.__strawberry_definition__.fields
    assert a_name.origin == a_mod.ABase
    assert a_age.origin == a_mod.AObject
    assert a_is_of_full_age.origin == a_mod.AObject
    assert b_name.origin == b_mod.BBase
    assert b_age.origin == b_mod.BObject
    assert b_is_of_full_age.origin == b_mod.BObject


def test_c_inheritance_resolver():
    @strawberry.type
    class Query:
        c: list[c_mod.CInheritance] = strawberry.field(
            resolver=c_mod.c_inheritance_resolver
        )

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[c_mod.CInheritance]


def test_c_inheritance_typeless_resolver():
    @strawberry.type
    class Query:
        c: list[c_mod.CInheritance] = strawberry.field(resolver=x_mod.typeless_resolver)

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[c_mod.CInheritance]


def test_c_inheritance_resolver_only():
    @strawberry.type
    class Query:
        c = strawberry.field(resolver=c_mod.c_inheritance_resolver)

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[c_mod.CInheritance]


def test_c_composition_resolver():
    @strawberry.type
    class Query:
        c: list[c_mod.CComposition] = strawberry.field(
            resolver=c_mod.c_composition_resolver
        )

    [field] = Query.__strawberry_definition__.fields
    assert field.type == list[c_mod.CComposition]
    [a_field, b_field] = field.type.of_type.__strawberry_definition__.fields
    assert a_field.type == list[a_mod.AObject]
    assert b_field.type == list[b_mod.BObject]


def test_c_composition_by_name_with_resolvers():
    [
        a_field,
        b_field,
    ] = c_mod.CCompositionByNameWithResolvers.__strawberry_definition__.fields
    assert a_field.type == list[a_mod.AObject]
    assert b_field.type == list[b_mod.BObject]


def test_c_composition_by_name_with_typeless_resolvers():
    [
        a_field,
        b_field,
    ] = c_mod.CCompositionByNameWithTypelessResolvers.__strawberry_definition__.fields
    assert a_field.type == list[a_mod.AObject]
    assert b_field.type == list[b_mod.BObject]


def test_c_composition_only_resolvers():
    [
        a_field,
        b_field,
    ] = c_mod.CCompositionOnlyResolvers.__strawberry_definition__.fields
    assert a_field.type == list[a_mod.AObject]
    assert b_field.type == list[b_mod.BObject]


def test_x_resolver():
    @strawberry.type
    class Query:
        c: list[a_mod.AObject] = strawberry.field(resolver=x_mod.typeless_resolver)

    [c_field] = Query.__strawberry_definition__.fields
    assert c_field.type == list[a_mod.AObject]



import pytest
from inline_snapshot import snapshot

import strawberry
from strawberry.schema.config import StrawberryConfig
from tests.conftest import skip_if_gql_32
from tests.schema.extensions.schema_extensions.conftest import (
    ExampleExtension,
)

pytestmark = skip_if_gql_32("GraphQL 3.3.0 is required for incremental execution")


@pytest.mark.xfail(reason="Not fully supported just yet")
async def test_basic_extension_with_defer(
    async_extension: type[ExampleExtension],
):
    @strawberry.type
    class Hero:
        id: strawberry.ID

        @strawberry.field
        async def name(self) -> str:
            return "Luke Skywalker"

    @strawberry.type
    class Query:
        @strawberry.field
        def hero(self) -> Hero:
            return Hero(id=strawberry.ID("1"))

    extension = async_extension()

    schema = strawberry.Schema(
        query=Query,
        extensions=[extension],
        config=StrawberryConfig(enable_experimental_incremental_execution=True),
    )

    result = await schema.execute(
        query="""
        query HeroNameQuery {
            hero {
                id
                ...NameFragment @defer
            }
        }
        fragment NameFragment on Hero {
            name
        }
        """,
    )

    initial = result.initial_result.formatted

    assert initial == snapshot(
        {
            "data": {"hero": {"id": "1"}},
            "hasNext": True,
            "pending": [{"path": ["hero"], "id": "0"}],
        }
    )

    async for subsequent in result.subsequent_results:
        assert subsequent.formatted == snapshot(
            {
                "completed": [
                    {
                        "id": "0",
                        "errors": [
                            {
                                "message": "String cannot represent value: <coroutine _async_resolver>",
                                "locations": [{"line": 9, "column": 13}],
                                "path": ["hero", "name"],
                            }
                        ],
                    }
                ],
                "hasNext": False,
            }
        )

    extension.assert_expected()



import re
import textwrap

import pytest

import strawberry
from strawberry.exceptions import InvalidSuperclassInterfaceError
from strawberry.printer import print_schema
from tests.conftest import skip_if_gql_32


def test_renaming_input_fields():
    @strawberry.input
    class FilterInput:
        in_: str | None = strawberry.field(name="in", default=strawberry.UNSET)

    @strawberry.type
    class Query:
        hello: str = "Hello"

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def filter(self, input: FilterInput) -> str:
            return f"Hello {input.in_ or 'nope'}"

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    query = "mutation { filter(input: {}) }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["filter"] == "Hello nope"


@skip_if_gql_32("formatting is different in gql 3.2")
def test_input_with_nonscalar_field_default():
    @strawberry.input
    class NonScalarField:
        id: int = 10
        nullable_field: int | None = None

    @strawberry.input
    class Input:
        non_scalar_field: NonScalarField = strawberry.field(
            default_factory=NonScalarField
        )
        id: int = 10

    @strawberry.type
    class ExampleOutput:
        input_id: int
        non_scalar_id: int
        non_scalar_nullable_field: int | None

    @strawberry.type
    class Query:
        @strawberry.field
        def example(self, data: Input) -> ExampleOutput:
            return ExampleOutput(
                input_id=data.id,
                non_scalar_id=data.non_scalar_field.id,
                non_scalar_nullable_field=data.non_scalar_field.nullable_field,
            )

    schema = strawberry.Schema(query=Query)

    expected = """
    type ExampleOutput {
      inputId: Int!
      nonScalarId: Int!
      nonScalarNullableField: Int
    }

    input Input {
      nonScalarField: NonScalarField! = { id: 10 }
      id: Int! = 10
    }

    input NonScalarField {
      id: Int! = 10
      nullableField: Int = null
    }

    type Query {
      example(data: Input!): ExampleOutput!
    }
    """
    assert print_schema(schema) == textwrap.dedent(expected).strip()

    query = """
    query($input_data: Input!)
    {
        example(data: $input_data) {
            inputId nonScalarId nonScalarNullableField
        }
    }
    """
    result = schema.execute_sync(
        query, variable_values={"input_data": {"nonScalarField": {}}}
    )

    assert not result.errors
    expected_result = {"inputId": 10, "nonScalarId": 10, "nonScalarNullableField": None}
    assert result.data["example"] == expected_result


@pytest.mark.raises_strawberry_exception(
    InvalidSuperclassInterfaceError,
    match=re.escape(
        "Input class 'SomeInput' cannot inherit from interface(s): SomeInterface"
    ),
)
def test_input_cannot_inherit_from_interface():
    @strawberry.interface
    class SomeInterface:
        some_arg: str

    @strawberry.input
    class SomeInput(SomeInterface):
        another_arg: str


@pytest.mark.raises_strawberry_exception(
    InvalidSuperclassInterfaceError,
    match=re.escape(
        "Input class 'SomeOtherInput' cannot inherit from interface(s): SomeInterface, SomeOtherInterface"
    ),
)
def test_input_cannot_inherit_from_interfaces():
    @strawberry.interface
    class SomeInterface:
        some_arg: str

    @strawberry.interface
    class SomeOtherInterface:
        some_other_arg: str

    @strawberry.input
    class SomeOtherInput(SomeInterface, SomeOtherInterface):
        another_arg: str



from __future__ import annotations

from typing import Any

from flask import Flask
from flask import Request as FlaskRequest
from flask import Response as FlaskResponse
from strawberry.flask.views import AsyncGraphQLView as BaseAsyncGraphQLView
from strawberry.http import GraphQLHTTPResponse
from strawberry.http.ides import GraphQL_IDE
from strawberry.schema import Schema
from strawberry.types import ExecutionResult
from tests.http.context import get_context
from tests.views.schema import Query

from .base import ResultOverrideFunction
from .flask import FlaskHttpClient


class GraphQLView(BaseAsyncGraphQLView[dict[str, object], object]):
    methods = ["GET", "POST", "PUT", "PATCH", "DELETE", "HEAD"]

    result_override: ResultOverrideFunction = None

    def __init__(self, *args: Any, **kwargs: Any):
        self.result_override = kwargs.pop("result_override")
        super().__init__(*args, **kwargs)

    async def get_root_value(self, request: FlaskRequest) -> Query:
        await super().get_root_value(request)  # for coverage
        return Query()

    async def get_context(
        self, request: FlaskRequest, response: FlaskResponse
    ) -> dict[str, object]:
        context = await super().get_context(request, response)

        return get_context(context)

    async def process_result(
        self, request: FlaskRequest, result: ExecutionResult
    ) -> GraphQLHTTPResponse:
        if self.result_override:
            return self.result_override(result)

        return await super().process_result(request, result)


class AsyncFlaskHttpClient(FlaskHttpClient):
    def __init__(
        self,
        schema: Schema,
        graphql_ide: GraphQL_IDE | None = "graphiql",
        allow_queries_via_get: bool = True,
        result_override: ResultOverrideFunction = None,
        multipart_uploads_enabled: bool = False,
    ):
        self.app = Flask(__name__)
        self.app.debug = True

        view = GraphQLView.as_view(
            "graphql_view",
            schema=schema,
            graphql_ide=graphql_ide,
            allow_queries_via_get=allow_queries_via_get,
            result_override=result_override,
            multipart_uploads_enabled=multipart_uploads_enabled,
        )

        self.app.add_url_rule(
            "/graphql",
            view_func=view,
        )



import strawberry
from strawberry.types.cast import get_strawberry_type_cast


def test_cast():
    @strawberry.type
    class SomeType: ...

    class OtherType: ...

    obj = OtherType
    assert get_strawberry_type_cast(obj) is None

    cast_obj = strawberry.cast(SomeType, obj)
    assert cast_obj is obj
    assert get_strawberry_type_cast(cast_obj) is SomeType


def test_cast_none_obj():
    @strawberry.type
    class SomeType: ...

    obj = None
    assert get_strawberry_type_cast(obj) is None

    cast_obj = strawberry.cast(SomeType, obj)
    assert cast_obj is None
    assert get_strawberry_type_cast(obj) is None



import pytest

from strawberry.experimental.pydantic._compat import IS_PYDANTIC_V2

needs_pydantic_v2 = pytest.mark.skipif(
    not IS_PYDANTIC_V2, reason="requires Pydantic v2"
)
needs_pydantic_v1 = pytest.mark.skipif(IS_PYDANTIC_V2, reason="requires Pydantic v1")



import re
import textwrap
import typing

import pytest

import strawberry
from strawberry.exceptions import StrawberryGraphQLError
from strawberry.exceptions.permission_fail_silently_requires_optional import (
    PermissionFailSilentlyRequiresOptionalError,
)
from strawberry.permission import BasePermission, PermissionExtension
from strawberry.printer import print_schema
from strawberry.utils.aio import aclosing


def test_raises_graphql_error_when_permission_method_is_missing():
    class IsAuthenticated(BasePermission):
        pass

    error_msg = (
        re.escape("Can't instantiate abstract class IsAuthenticated ") + r"(.*)*"
    )

    with pytest.raises(TypeError, match=error_msg):

        @strawberry.type
        class Query:
            @strawberry.field(permission_classes=[IsAuthenticated])
            def user(self) -> str:  # pragma: no cover
                return "patrick"


def test_raises_graphql_error_when_permission_is_denied():
    class IsAuthenticated(BasePermission):
        message = "User is not authenticated"

        def has_permission(
            self, source: typing.Any, info: strawberry.Info, **kwargs: typing.Any
        ) -> bool:
            return False

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthenticated])
        def user(self) -> str:  # pragma: no cover
            return "patrick"

    schema = strawberry.Schema(query=Query)

    query = "{ user }"

    result = schema.execute_sync(query)
    assert result.errors[0].message == "User is not authenticated"


@pytest.mark.asyncio
async def test_raises_permission_error_for_subscription():
    class IsAdmin(BasePermission):
        message = "You are not authorized"

        def has_permission(
            self, source: typing.Any, info: strawberry.Info, **kwargs: typing.Any
        ) -> bool:
            return False

    @strawberry.type
    class Query:
        name: str = "Andrew"

    @strawberry.type
    class Subscription:
        @strawberry.subscription(permission_classes=[IsAdmin])
        async def user(self) -> typing.AsyncGenerator[str, None]:  # pragma: no cover
            yield "Hello"

    schema = strawberry.Schema(query=Query, subscription=Subscription)

    query = "subscription { user }"

    async with aclosing(await schema.subscribe(query)) as sub_result:
        result = await sub_result.__anext__()

    assert result.errors[0].message == "You are not authorized"


@pytest.mark.asyncio
async def test_sync_permissions_work_with_async_resolvers():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return info.context["user"] == "Patrick"

    @strawberry.type
    class User:
        name: str
        email: str

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        async def user(self, name: str) -> User:
            return User(name=name, email="patrick.arminio@gmail.com")

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email } }'
    result = await schema.execute(query, context_value={"user": "Patrick"})
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "marco") { email } }'
    result = await schema.execute(query, context_value={"user": "Marco"})
    assert result.errors[0].message == "User is not authorized"


def test_can_use_source_when_testing_permission():
    class CanSeeEmail(BasePermission):
        message = "Cannot see email for this user"

        def has_permission(
            self, source: typing.Any, info: strawberry.Info, **kwargs: typing.Any
        ) -> bool:
            return source.name.lower() == "patrick"

    @strawberry.type
    class User:
        name: str

        @strawberry.field(permission_classes=[CanSeeEmail])
        def email(self) -> str:
            return "patrick.arminio@gmail.com"

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self, name: str) -> User:
            return User(name=name)

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email } }'

    result = schema.execute_sync(query)
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "marco") { email } }'

    result = schema.execute_sync(query)
    assert result.errors[0].message == "Cannot see email for this user"


def test_can_use_args_when_testing_permission():
    class CanSeeEmail(BasePermission):
        message = "Cannot see email for this user"

        def has_permission(
            self, source: typing.Any, info: strawberry.Info, **kwargs: typing.Any
        ) -> bool:
            return kwargs.get("secure", False)

    @strawberry.type
    class User:
        name: str

        @strawberry.field(permission_classes=[CanSeeEmail])
        def email(self, secure: bool) -> str:
            return "patrick.arminio@gmail.com"

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self, name: str) -> User:
            return User(name=name)

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email(secure: true) } }'

    result = schema.execute_sync(query)
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "patrick") { email(secure: false) } }'

    result = schema.execute_sync(query)
    assert result.errors[0].message == "Cannot see email for this user"


def test_can_use_on_simple_fields():
    class CanSeeEmail(BasePermission):
        message = "Cannot see email for this user"

        def has_permission(
            self, source: typing.Any, info: strawberry.Info, **kwargs: typing.Any
        ) -> bool:
            return source.name.lower() == "patrick"

    @strawberry.type
    class User:
        name: str
        email: str = strawberry.field(permission_classes=[CanSeeEmail])

    @strawberry.type
    class Query:
        @strawberry.field
        def user(self, name: str) -> User:
            return User(name=name, email="patrick.arminio@gmail.com")

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email } }'

    result = schema.execute_sync(query)
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "marco") { email } }'

    result = schema.execute_sync(query)
    assert result.errors[0].message == "Cannot see email for this user"


@pytest.mark.asyncio
async def test_dataclass_field_with_async_permission_class():
    class CanSeeEmail(BasePermission):
        message = "Cannot see email for this user"

        async def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return source.name.lower() == "patrick"

    @strawberry.type
    class User:
        name: str
        email: str = strawberry.field(permission_classes=[CanSeeEmail])

    @strawberry.type
    class Query:
        @strawberry.field()
        async def user(self, name: str) -> User:
            return User(name=name, email="patrick.arminio@gmail.com")

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email } }'
    result = await schema.execute(query)
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "marco") { email } }'
    result = await schema.execute(query)
    assert result.errors[0].message == "Cannot see email for this user"


@pytest.mark.asyncio
async def test_async_resolver_with_async_permission_class():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        async def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return info.context["user"] == "Patrick"

    @strawberry.type
    class User:
        name: str
        email: str

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        async def user(self, name: str) -> User:
            return User(name=name, email="patrick.arminio@gmail.com")

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email } }'
    result = await schema.execute(query, context_value={"user": "Patrick"})
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "marco") { email } }'
    result = await schema.execute(query, context_value={"user": "Marco"})
    assert result.errors[0].message == "User is not authorized"


@pytest.mark.asyncio
async def test_sync_resolver_with_async_permission_class():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        async def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return info.context["user"] == "Patrick"

    @strawberry.type
    class User:
        name: str
        email: str

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        def user(self, name: str) -> User:
            return User(name=name, email="patrick.arminio@gmail.com")

    schema = strawberry.Schema(query=Query)

    query = '{ user(name: "patrick") { email } }'
    result = await schema.execute(query, context_value={"user": "Patrick"})
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"

    query = '{ user(name: "marco") { email } }'
    result = await schema.execute(query, context_value={"user": "Marco"})
    assert result.errors[0].message == "User is not authorized"


@pytest.mark.asyncio
async def test_mixed_sync_and_async_permission_classes():
    class IsAuthorizedAsync(BasePermission):
        message = "User is not authorized (async)"

        async def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return info.context.get("passAsync", False)

    class IsAuthorizedSync(BasePermission):
        message = "User is not authorized (sync)"

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return info.context.get("passSync", False)

    @strawberry.type
    class User:
        name: str
        email: str

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorizedAsync, IsAuthorizedSync])
        def user(self, name: str) -> User:
            return User(name=name, email="patrick.arminio@gmail.com")

    schema = strawberry.Schema(query=Query)
    query = '{ user(name: "patrick") { email } }'

    context = {"passAsync": False, "passSync": False}
    result = await schema.execute(query, context_value=context)
    assert result.errors[0].message == "User is not authorized (async)"

    context = {"passAsync": True, "passSync": False}
    result = await schema.execute(query, context_value=context)
    assert result.errors[0].message == "User is not authorized (sync)"

    context = {"passAsync": False, "passSync": True}
    result = await schema.execute(query, context_value=context)
    assert result.errors[0].message == "User is not authorized (async)"

    context = {"passAsync": True, "passSync": True}
    result = await schema.execute(query, context_value=context)
    assert result.data["user"]["email"] == "patrick.arminio@gmail.com"


def test_permissions_with_custom_extensions():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"
        error_extensions = {"code": "UNAUTHORIZED"}

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return False

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        def name(self) -> str:  # pragma: no cover
            return "ABC"

    schema = strawberry.Schema(query=Query)
    query = "{ name }"

    result = schema.execute_sync(query)
    assert result.errors[0].message == "User is not authorized"
    assert result.errors[0].extensions
    assert result.errors[0].extensions["code"] == "UNAUTHORIZED"


def test_permissions_with_custom_extensions_on_custom_error():
    class CustomError(StrawberryGraphQLError):
        def __init__(self, message: str):
            super().__init__(message, extensions={"general_info": "CUSTOM_ERROR"})

    class IsAuthorized(BasePermission):
        message = "User is not authorized"
        error_class = CustomError
        error_extensions = {"code": "UNAUTHORIZED"}

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return False

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        def name(self) -> str:  # pragma: no cover
            return "ABC"

    schema = strawberry.Schema(query=Query)
    query = "{ name }"

    result = schema.execute_sync(query)

    assert result.errors[0].message == "User is not authorized"
    assert result.errors[0].extensions
    assert result.errors[0].extensions["code"] == "UNAUTHORIZED"
    assert result.errors[0].extensions["general_info"] == "CUSTOM_ERROR"


def test_silent_permissions_optional():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return False

    @strawberry.type
    class Query:
        @strawberry.field(
            extensions=[PermissionExtension([IsAuthorized()], fail_silently=True)]
        )
        def name(self) -> str | None:  # pragma: no cover
            return "ABC"

    schema = strawberry.Schema(query=Query)
    query = "{ name }"
    result = schema.execute_sync(query)

    assert result.data["name"] is None
    assert result.errors is None


def test_silent_permissions_optional_list():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return False

    @strawberry.type
    class Query:
        @strawberry.field(
            extensions=[PermissionExtension([IsAuthorized()], fail_silently=True)]
        )
        def names(self) -> list[str] | None:  # pragma: no cover
            return ["ABC"]

    schema = strawberry.Schema(query=Query)
    query = "{ names }"
    result = schema.execute_sync(query)

    assert result.data["names"] == []
    assert result.errors is None


def test_silent_permissions_list():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(self, source, info, **kwargs: typing.Any) -> bool:
            return False

    @strawberry.type
    class Query:
        @strawberry.field(
            extensions=[PermissionExtension([IsAuthorized()], fail_silently=True)]
        )
        def names(self) -> list[str]:  # pragma: no cover
            return ["ABC"]

    schema = strawberry.Schema(query=Query)
    query = "{ names }"
    result = schema.execute_sync(query)

    assert result.data["names"] == []
    assert result.errors is None


@pytest.mark.raises_strawberry_exception(
    PermissionFailSilentlyRequiresOptionalError,
    match="Cannot use fail_silently=True with a non-optional or non-list field",
)
def test_silent_permissions_incompatible_types():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(
            self, source, info, **kwargs: typing.Any
        ) -> bool:  # pragma: no cover
            return False

    @strawberry.type
    class User:
        name: str

    @strawberry.type
    class Query:
        @strawberry.field(
            extensions=[PermissionExtension([IsAuthorized()], fail_silently=True)]
        )
        def name(self) -> User:  # pragma: no cover
            return User(name="ABC")

    strawberry.Schema(query=Query)


def test_permission_directives_added():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(
            self, source, info, **kwargs: typing.Any
        ) -> bool:  # pragma: no cover
            return False

    @strawberry.type
    class Query:
        @strawberry.field(extensions=[PermissionExtension([IsAuthorized()])])
        def name(self) -> str:  # pragma: no cover
            return "ABC"

    schema = strawberry.Schema(query=Query)

    expected_output = """
    directive @isAuthorized on FIELD_DEFINITION

    type Query {
      name: String! @isAuthorized
    }
    """
    assert print_schema(schema) == textwrap.dedent(expected_output).strip()


def test_permission_directives_not_added_on_field():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(
            self, source, info, **kwargs: typing.Any
        ) -> bool:  # pragma: no cover
            return False

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        def name(self) -> str:  # pragma: no cover
            return "ABC"

    schema = strawberry.Schema(query=Query)

    expected_output = """
    type Query {
      name: String!
    }
    """
    assert print_schema(schema) == textwrap.dedent(expected_output).strip()


def test_basic_permission_access_inputs():
    class IsAuthorized(BasePermission):
        message = "User is not authorized"

        def has_permission(
            self, source, info, **kwargs: typing.Any
        ) -> bool:  # pragma: no cover
            return kwargs["a_key"] == "secret"

    @strawberry.type
    class Query:
        @strawberry.field(permission_classes=[IsAuthorized])
        def name(self, a_key: str) -> str:  # pragma: no cover
            return "Erik"

    schema = strawberry.Schema(query=Query)

    query = '{ name(aKey: "example") }'
    result = schema.execute_sync(query)

    assert result.errors[0].message == "User is not authorized"

    query = '{ name(aKey: "secret") }'

    result = schema.execute_sync(query)

    assert result.data["name"] == "Erik"



from graphql import located_error

import strawberry
from strawberry.types import Info


def test_fetch_entities():
    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str

        @classmethod
        def resolve_reference(cls, upc) -> "Product":
            return Product(upc=upc)

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [{"__typename": "Product", "upc": "B00005N5PF"}]
        },
    )

    assert not result.errors

    assert result.data == {"_entities": [{"upc": "B00005N5PF"}]}


def test_info_param_in_resolve_reference():
    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str
        debug_field_name: str

        @classmethod
        def resolve_reference(cls, info: strawberry.Info, upc: str) -> "Product":
            return Product(upc=upc, debug_field_name=info.field_name)

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                    debugFieldName
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [{"__typename": "Product", "upc": "B00005N5PF"}]
        },
    )

    assert not result.errors

    assert result.data == {
        "_entities": [
            {
                "upc": "B00005N5PF",
                # _entities is the field that's called by federation
                "debugFieldName": "_entities",
            }
        ]
    }


def test_does_not_need_custom_resolve_reference_for_basic_things():
    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [{"__typename": "Product", "upc": "B00005N5PF"}]
        },
    )

    assert not result.errors

    assert result.data == {"_entities": [{"upc": "B00005N5PF"}]}


def test_does_not_need_custom_resolve_reference_nested():
    @strawberry.federation.type(keys=["id"])
    class Something:
        id: str

    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str
        something: Something

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                    something {
                        id
                    }
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [
                {"__typename": "Product", "upc": "B00005N5PF", "something": {"id": "1"}}
            ]
        },
    )

    assert not result.errors

    assert result.data == {
        "_entities": [{"upc": "B00005N5PF", "something": {"id": "1"}}]
    }


def test_fails_properly_when_wrong_key_is_passed():
    @strawberry.type
    class Something:
        id: str

    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str
        something: Something

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                    something {
                        id
                    }
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [{"__typename": "Product", "not_upc": "B00005N5PF"}]
        },
    )

    assert result.errors

    assert result.errors[0].message == "Unable to resolve reference for Product"


def test_fails_properly_when_wrong_data_is_passed():
    @strawberry.federation.type(keys=["id"])
    class Something:
        id: str

    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str
        something: Something

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                    something {
                        id
                    }
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [
                {
                    "__typename": "Product",
                    "upc": "B00005N5PF",
                    "not_something": {"id": "1"},
                }
            ]
        },
    )

    assert result.errors

    assert result.errors[0].message == "Unable to resolve reference for Product"


def test_propagates_original_error_message_with_auto_graphql_error_metadata():
    @strawberry.federation.type(keys=["id"])
    class Product:
        id: strawberry.ID

        @classmethod
        def resolve_reference(cls, id: strawberry.ID) -> "Product":
            raise Exception("Foo bar")

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def mock(self) -> Product | None:
            return None

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    id
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [
                {
                    "__typename": "Product",
                    "id": "B00005N5PF",
                }
            ]
        },
    )

    assert len(result.errors) == 1
    error = result.errors[0].formatted
    assert error["message"] == "Foo bar"
    assert error["path"] == ["_entities", 0]
    assert error["locations"] == [{"column": 13, "line": 3}]
    assert "extensions" not in error


def test_propagates_custom_type_error_message_with_auto_graphql_error_metadata():
    class MyTypeError(TypeError):
        pass

    @strawberry.federation.type(keys=["id"])
    class Product:
        id: strawberry.ID

        @classmethod
        def resolve_reference(cls, id: strawberry.ID) -> "Product":
            raise MyTypeError("Foo bar")

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def mock(self) -> Product | None:
            return None

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    id
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [
                {
                    "__typename": "Product",
                    "id": "B00005N5PF",
                }
            ]
        },
    )

    assert len(result.errors) == 1
    error = result.errors[0].formatted
    assert error["message"] == "Foo bar"
    assert error["path"] == ["_entities", 0]
    assert error["locations"] == [{"column": 13, "line": 3}]
    assert "extensions" not in error


def test_propagates_original_error_message_and_graphql_error_metadata():
    @strawberry.federation.type(keys=["id"])
    class Product:
        id: strawberry.ID

        @classmethod
        def resolve_reference(cls, info: Info, id: strawberry.ID) -> "Product":
            exception = Exception("Foo bar")
            exception.extensions = {"baz": "qux"}
            raise located_error(
                exception,
                nodes=info._raw_info.field_nodes[0],
                path=["_entities_override", 0],
            )

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def mock(self) -> Product | None:
            return None

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    id
                }
            }
        }
    """

    result = schema.execute_sync(
        query,
        variable_values={
            "representations": [
                {
                    "__typename": "Product",
                    "id": "B00005N5PF",
                }
            ]
        },
    )

    assert len(result.errors) == 1
    error = result.errors[0].formatted
    assert error["message"] == "Foo bar"
    assert error["path"] == ["_entities_override", 0]
    assert error["locations"] == [{"column": 13, "line": 3}]
    assert error["extensions"] == {"baz": "qux"}


async def test_can_use_async_resolve_reference():
    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str

        @classmethod
        async def resolve_reference(cls, upc: str) -> "Product":
            return Product(upc=upc)

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                }
            }
        }
    """

    result = await schema.execute(
        query,
        variable_values={
            "representations": [{"__typename": "Product", "upc": "B00005N5PF"}]
        },
    )

    assert not result.errors

    assert result.data == {"_entities": [{"upc": "B00005N5PF"}]}


async def test_can_use_async_resolve_reference_multiple_representations():
    @strawberry.federation.type(keys=["upc"])
    class Product:
        upc: str

        @classmethod
        async def resolve_reference(cls, upc: str) -> "Product":
            return Product(upc=upc)

    @strawberry.federation.type(extend=True)
    class Query:
        @strawberry.field
        def top_products(self, first: int) -> list[Product]:  # pragma: no cover
            return []

    schema = strawberry.federation.Schema(query=Query)

    query = """
        query ($representations: [_Any!]!) {
            _entities(representations: $representations) {
                ... on Product {
                    upc
                }
            }
        }
    """

    result = await schema.execute(
        query,
        variable_values={
            "representations": [
                {"__typename": "Product", "upc": "B00005N5PF"},
                {"__typename": "Product", "upc": "B00005N5PG"},
            ]
        },
    )

    assert not result.errors

    assert result.data == {"_entities": [{"upc": "B00005N5PF"}, {"upc": "B00005N5PG"}]}



import ast
import textwrap
from pathlib import Path

from pytest_snapshot.plugin import Snapshot

from strawberry.schema_codegen import codegen

HERE = Path(__file__).parent


def test_long_descriptions(snapshot: Snapshot):
    snapshot.snapshot_dir = HERE / "snapshots"

    schema = '''
    """A connection to a list of items."""
    type FilmCharactersConnection {
    """Information to aid in pagination."""
    pageInfo: PageInfo!

    """A list of edges."""
    edges: [FilmCharactersEdge]

    """
    A count of the total number of objects in this connection, ignoring pagination.
    This allows a client to fetch the first five objects by passing "5" as the
    argument to "first", then fetch the total count so it could display "5 of 83",
    for example.
    """
    totalCount: Int

    """
    A list of all of the objects returned in the connection. This is a convenience
    field provided for quickly exploring the API; rather than querying for
    "{ edges { node } }" when no edge data is needed, this field can be be used
    instead. Note that when clients like Relay need to fetch the "cursor" field on
    the edge to enable efficient pagination, this shortcut cannot be used, and the
    full "{ edges { node } }" version should be used instead.
    """
    characters: [Person]
    }
    '''

    output = codegen(schema)

    ast.parse(output)

    snapshot.assert_match(output, "long_descriptions.py")


def test_can_convert_descriptions_with_quotes():
    schema = '''
    """A type of person or character within the "Star Wars" Universe."""
    type Species {
        """The classification of this species, such as "mammal" or "reptile"."""
        classification: String!
    }
    '''

    output = codegen(schema)

    expected_output = textwrap.dedent(
        """
        from __future__ import annotations
        import strawberry

        @strawberry.type(description='A type of person or character within the "Star Wars" Universe.')
        class Species:
            classification: str = strawberry.field(description='The classification of this species, such as "mammal" or "reptile".')
        """
    ).lstrip()

    assert output == expected_output



from pathlib import Path

from inline_snapshot import snapshot
from typer import Typer
from typer.testing import CliRunner

from tests.typecheckers.utils.marks import skip_on_windows

pytestmark = skip_on_windows


def _simplify_path(path: str) -> str:
    path = Path(path)

    root = Path(__file__).parents[1]

    return str(path.relative_to(root))


def test_find_model_name(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:schema"
    result = cli_runner.invoke(cli_app, ["locate-definition", selector, "User"])

    assert result.exit_code == 0
    assert _simplify_path(result.stdout.strip()) == snapshot(
        "fixtures/sample_package/sample_module.py:38:7"
    )


def test_find_model_field(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:schema"
    result = cli_runner.invoke(cli_app, ["locate-definition", selector, "User.name"])

    assert result.exit_code == 0
    assert _simplify_path(result.stdout.strip()) == snapshot(
        "fixtures/sample_package/sample_module.py:39:5"
    )


def test_find_missing_model(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:schema"
    result = cli_runner.invoke(cli_app, ["locate-definition", selector, "Missing"])

    assert result.exit_code == 1
    assert result.stderr.strip() == snapshot("Definition not found: Missing")


def test_find_missing_model_field(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:schema"
    result = cli_runner.invoke(
        cli_app, ["locate-definition", selector, "Missing.field"]
    )

    assert result.exit_code == 1
    assert result.stderr.strip() == snapshot("Definition not found: Missing.field")


def test_find_missing_schema(cli_app: Typer, cli_runner: CliRunner):
    selector = "tests.fixtures.sample_package.sample_module:missing"
    result = cli_runner.invoke(cli_app, ["locate-definition", selector, "User"])

    assert result.exit_code == 2



from typing import Union

class PartialBlogPost:
    # typename: BlogPost
    title: str

class OperationNameResultInterfaceBlogPost:
    # typename: BlogPost
    id: str
    title: str

class OperationNameResultInterfaceImage:
    # typename: Image
    id: str
    url: str

OperationNameResultInterface = Union[OperationNameResultInterfaceBlogPost, OperationNameResultInterfaceImage]

class OperationNameResult:
    interface: OperationNameResultInterface



from enum import Enum
from textwrap import dedent
from typing import Annotated

import pytest

import strawberry
from strawberry.types.lazy_type import lazy


def test_enum_resolver():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    @strawberry.type
    class Query:
        @strawberry.field
        def best_flavour(self) -> IceCreamFlavour:
            return IceCreamFlavour.STRAWBERRY

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavour }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["bestFlavour"] == "STRAWBERRY"

    @strawberry.type
    class Cone:
        flavour: IceCreamFlavour

    @strawberry.type
    class Query:
        @strawberry.field
        def cone(self) -> Cone:
            return Cone(flavour=IceCreamFlavour.STRAWBERRY)

    schema = strawberry.Schema(query=Query)

    query = "{ cone { flavour } }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["cone"]["flavour"] == "STRAWBERRY"


def test_enum_arguments():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    @strawberry.type
    class Query:
        @strawberry.field
        def flavour_available(self, flavour: IceCreamFlavour) -> bool:
            return flavour == IceCreamFlavour.STRAWBERRY

    @strawberry.input
    class ConeInput:
        flavour: IceCreamFlavour

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def eat_cone(self, input: ConeInput) -> bool:
            return input.flavour == IceCreamFlavour.STRAWBERRY

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    query = "{ flavourAvailable(flavour: VANILLA) }"
    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["flavourAvailable"] is False

    query = "{ flavourAvailable(flavour: STRAWBERRY) }"
    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["flavourAvailable"] is True

    query = "mutation { eatCone(input: { flavour: VANILLA }) }"
    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["eatCone"] is False

    query = "mutation { eatCone(input: { flavour: STRAWBERRY }) }"
    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["eatCone"] is True


def test_lazy_enum_arguments():
    LazyEnum = Annotated[
        "LazyEnum", lazy("tests.schema.test_lazy_types.test_lazy_enums")
    ]

    @strawberry.type
    class Query:
        @strawberry.field
        def something(self, enum: LazyEnum) -> LazyEnum:
            return enum

    schema = strawberry.Schema(query=Query)

    query = "{ something(enum: BREAD) }"
    result = schema.execute_sync(query)
    assert not result.errors
    assert result.data["something"] == "BREAD"


def test_enum_falsy_values():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = ""
        STRAWBERRY = 0

    @strawberry.input
    class Input:
        flavour: IceCreamFlavour
        optional_flavour: IceCreamFlavour | None = None

    @strawberry.type
    class Query:
        @strawberry.field
        def print_flavour(self, input: Input) -> str:
            return f"{input.flavour.value}"

    schema = strawberry.Schema(query=Query)

    query = "{ printFlavour(input: { flavour: VANILLA }) }"
    result = schema.execute_sync(query)

    assert not result.errors
    assert not result.data["printFlavour"]

    query = "{ printFlavour(input: { flavour: STRAWBERRY }) }"
    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["printFlavour"] == "0"


def test_enum_in_list():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"
        PISTACHIO = "pistachio"

    @strawberry.type
    class Query:
        @strawberry.field
        def best_flavours(self) -> list[IceCreamFlavour]:
            return [IceCreamFlavour.STRAWBERRY, IceCreamFlavour.PISTACHIO]

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavours }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["bestFlavours"] == ["STRAWBERRY", "PISTACHIO"]


def test_enum_in_optional_list():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"
        PISTACHIO = "pistachio"

    @strawberry.type
    class Query:
        @strawberry.field
        def best_flavours(self) -> list[IceCreamFlavour] | None:
            return None

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavours }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["bestFlavours"] is None


@pytest.mark.asyncio
async def test_enum_resolver_async():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    @strawberry.type
    class Query:
        @strawberry.field
        async def best_flavour(self) -> IceCreamFlavour:
            return IceCreamFlavour.STRAWBERRY

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavour }"

    result = await schema.execute(query)

    assert not result.errors
    assert result.data["bestFlavour"] == "STRAWBERRY"


@pytest.mark.asyncio
async def test_enum_in_list_async():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"
        PISTACHIO = "pistachio"

    @strawberry.type
    class Query:
        @strawberry.field
        async def best_flavours(self) -> list[IceCreamFlavour]:
            return [IceCreamFlavour.STRAWBERRY, IceCreamFlavour.PISTACHIO]

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavours }"

    result = await schema.execute(query)

    assert not result.errors
    assert result.data["bestFlavours"] == ["STRAWBERRY", "PISTACHIO"]


def test_enum_as_argument():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"
        PISTACHIO = "pistachio"

    @strawberry.type
    class Query:
        @strawberry.field
        def create_flavour(self, flavour: IceCreamFlavour) -> str:
            return f"{flavour.name}"

    schema = strawberry.Schema(query=Query)

    expected = dedent(
        """
        enum IceCreamFlavour {
          VANILLA
          STRAWBERRY
          CHOCOLATE
          PISTACHIO
        }

        type Query {
          createFlavour(flavour: IceCreamFlavour!): String!
        }
        """
    ).strip()

    assert str(schema) == expected

    query = "{ createFlavour(flavour: CHOCOLATE) }"
    result = schema.execute_sync(query)
    assert not result.errors
    assert result.data["createFlavour"] == "CHOCOLATE"

    # Explicitly using `variable_values` now so that the enum is parsed using
    # `CustomGraphQLEnumType.parse_value()` instead of `.parse_literal`
    query = "query ($flavour: IceCreamFlavour!) { createFlavour(flavour: $flavour) }"
    result = schema.execute_sync(query, variable_values={"flavour": "VANILLA"})
    assert not result.errors
    assert result.data["createFlavour"] == "VANILLA"


def test_enum_as_default_argument():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"
        PISTACHIO = "pistachio"

    @strawberry.type
    class Query:
        @strawberry.field
        def create_flavour(
            self, flavour: IceCreamFlavour = IceCreamFlavour.STRAWBERRY
        ) -> str:
            return f"{flavour.name}"

    schema = strawberry.Schema(query=Query)

    expected = dedent(
        """
        enum IceCreamFlavour {
          VANILLA
          STRAWBERRY
          CHOCOLATE
          PISTACHIO
        }

        type Query {
          createFlavour(flavour: IceCreamFlavour! = STRAWBERRY): String!
        }
        """
    ).strip()

    assert str(schema) == expected

    query = "{ createFlavour }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["createFlavour"] == "STRAWBERRY"


def test_enum_resolver_plain_value():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = "strawberry"
        CHOCOLATE = "chocolate"

    @strawberry.type
    class Query:
        @strawberry.field
        def best_flavour(self) -> IceCreamFlavour:
            return "strawberry"  # type: ignore

    schema = strawberry.Schema(query=Query)

    query = "{ bestFlavour }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data["bestFlavour"] == "STRAWBERRY"


def test_enum_deprecated_value():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        STRAWBERRY = strawberry.enum_value(
            "strawberry", deprecation_reason="We ran out"
        )
        CHOCOLATE = strawberry.enum_value("chocolate")

    @strawberry.type
    class Query:
        @strawberry.field
        def best_flavour(self) -> IceCreamFlavour:
            return IceCreamFlavour.STRAWBERRY

    schema = strawberry.Schema(query=Query)

    query = """
    {
        __type(name: "IceCreamFlavour") {
            enumValues(includeDeprecated: true) {
                name
                isDeprecated
                deprecationReason
            }
        }
    }
    """

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["__type"]["enumValues"] == [
        {"deprecationReason": None, "isDeprecated": False, "name": "VANILLA"},
        {"deprecationReason": "We ran out", "isDeprecated": True, "name": "STRAWBERRY"},
        {"deprecationReason": None, "isDeprecated": False, "name": "CHOCOLATE"},
    ]


def test_can_use_enum_values_in_input():
    @strawberry.enum
    class TestEnum(Enum):
        A = "A"
        B = strawberry.enum_value("B")
        C = strawberry.enum_value("Coconut", deprecation_reason="We ran out")

    @strawberry.type
    class Query:
        @strawberry.field
        def receive_enum(self, test: TestEnum) -> str:
            return str(test)

    schema = strawberry.Schema(query=Query)

    query = """
    query {
        a: receiveEnum(test: A)
        b: receiveEnum(test: B)
        c: receiveEnum(test: C)
    }
    """

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["a"] == "TestEnum.A"
    assert result.data["b"] == "TestEnum.B"
    assert result.data["c"] == "TestEnum.C"


def test_can_give_custom_name_to_enum_value():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        CHOCOLATE_COOKIE = strawberry.enum_value("chocolate", name="chocolateCookie")

    @strawberry.type
    class Query:
        @strawberry.field
        def best_flavour(self) -> IceCreamFlavour:
            return IceCreamFlavour.CHOCOLATE_COOKIE

    schema = strawberry.Schema(query=Query)

    assert (
        str(schema)
        == dedent(
            """
        enum IceCreamFlavour {
          VANILLA
          chocolateCookie
        }

        type Query {
          bestFlavour: IceCreamFlavour!
        }
        """
        ).strip()
    )

    query = """
    {
        bestFlavour
    }
    """

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["bestFlavour"] == "chocolateCookie"


def test_can_use_enum_with_custom_name_as_input():
    @strawberry.enum
    class IceCreamFlavour(Enum):
        VANILLA = "vanilla"
        CHOCOLATE_COOKIE = strawberry.enum_value("chocolate", name="chocolateCookie")

    @strawberry.type
    class Query:
        @strawberry.field
        def get_flavour(self, flavour: IceCreamFlavour) -> str:
            return flavour.name

    schema = strawberry.Schema(query=Query)

    assert (
        str(schema)
        == dedent(
            """
        enum IceCreamFlavour {
          VANILLA
          chocolateCookie
        }

        type Query {
          getFlavour(flavour: IceCreamFlavour!): String!
        }
        """
        ).strip()
    )

    query = """
    {
        getFlavour(flavour: chocolateCookie)
    }
    """

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data
    assert result.data["getFlavour"] == "CHOCOLATE_COOKIE"



from __future__ import annotations

import typing
from typing import ClassVar, ForwardRef, Optional, Union

from strawberry.scalars import JSON
from strawberry.utils.typing import eval_type, is_classvar


def test_eval_type():
    class Foo: ...

    assert eval_type(ForwardRef("Foo | None"), globals(), locals()) == Optional[Foo]
    assert eval_type(ForwardRef("Foo | str"), globals(), locals()) == Union[Foo, str]
    assert (
        eval_type(ForwardRef("Foo | str | None"), globals(), locals())
        == Union[Foo, str, None]
    )
    assert (
        eval_type(ForwardRef("list[Foo | str] | None"), globals(), locals())
        == Union[list[Foo | str], None]
    )
    assert (
        eval_type(ForwardRef("list[Foo | str] | None | int"), globals(), locals())
        == Union[list[Foo | str], int, None]
    )
    assert eval_type(ForwardRef("JSON | None"), globals(), locals()) == Optional[JSON]


def test_eval_type_generic_type_alias():
    class Foo: ...

    assert eval_type(ForwardRef("Foo | None"), globals(), locals()) == Optional[Foo]
    assert eval_type(ForwardRef("Foo | str"), globals(), locals()) == Union[Foo, str]
    assert (
        eval_type(ForwardRef("Foo | str | None"), globals(), locals())
        == Union[Foo, str, None]
    )
    assert (
        eval_type(ForwardRef("list[Foo | str] | None"), globals(), locals())
        == Union[list[Foo | str], None]  # type: ignore
    )
    assert (
        eval_type(ForwardRef("list[Foo | str] | None | int"), globals(), locals())
        == Union[list[Foo | str], int, None]  # type: ignore
    )


def test_is_classvar():
    class Foo:
        attr1: str
        attr2: ClassVar[str]
        attr3: typing.ClassVar[str]

    assert not is_classvar(Foo, Foo.__annotations__["attr1"])
    assert is_classvar(Foo, Foo.__annotations__["attr2"])
    assert is_classvar(Foo, Foo.__annotations__["attr3"])



import strawberry
from strawberry.extensions import DisableIntrospection


def test_disables_introspection():
    @strawberry.type
    class Query:
        hello: str

    schema = strawberry.Schema(
        query=Query,
        extensions=[DisableIntrospection()],
    )

    result = schema.execute_sync("query { __schema { __typename } }")
    assert result.data is None
    assert result.errors is not None

    formatted_errors = [err.formatted for err in result.errors]
    assert formatted_errors == [
        {
            "message": "GraphQL introspection has been disabled, but the requested query contained the field '__schema'.",
            "locations": [{"line": 1, "column": 9}],
        }
    ]


def test_does_not_affect_non_introspection_queries():
    @strawberry.type
    class Query:
        hello: str

    schema = strawberry.Schema(
        query=Query,
        extensions=[DisableIntrospection()],
    )

    result = schema.execute_sync("query { __typename }")
    assert result.data == {"__typename": "Query"}
    assert result.errors is None



import datetime

import pytest
from graphql import GraphQLError

import strawberry
from strawberry.types.execution import ExecutionResult


def test_serialization():
    @strawberry.type
    class Query:
        @strawberry.field
        def serialize(self) -> datetime.date:
            return datetime.date(2019, 10, 25)

    schema = strawberry.Schema(Query)

    result = schema.execute_sync("{ serialize }")

    assert not result.errors
    assert result.data["serialize"] == "2019-10-25"


def test_deserialization():
    @strawberry.type
    class Query:
        deserialized = None

        @strawberry.field
        def deserialize(self, arg: datetime.date) -> bool:
            Query.deserialized = arg
            return True

    schema = strawberry.Schema(Query)

    query = """query Deserialize($value: Date!) {
        deserialize(arg: $value)
    }"""
    result = schema.execute_sync(query, variable_values={"value": "2019-10-25"})

    assert not result.errors
    assert Query.deserialized == datetime.date(2019, 10, 25)


def test_deserialization_with_parse_literal():
    @strawberry.type
    class Query:
        deserialized = None

        @strawberry.field
        def deserialize(self, arg: datetime.date) -> bool:
            Query.deserialized = arg
            return True

    schema = strawberry.Schema(Query)

    query = """query Deserialize {
        deserialize(arg: "2019-10-25")
    }"""
    result = schema.execute_sync(query)

    assert not result.errors
    assert Query.deserialized == datetime.date(2019, 10, 25)


def execute_mutation(value) -> ExecutionResult:
    @strawberry.type
    class Query:
        ok: bool

    @strawberry.type
    class Mutation:
        @strawberry.mutation
        def date_input(self, date_input: datetime.date) -> datetime.date:
            assert isinstance(date_input, datetime.date)
            return date_input

    schema = strawberry.Schema(query=Query, mutation=Mutation)

    return schema.execute_sync(
        """
            mutation dateInput($value: Date!) {
                dateInput(dateInput: $value)
            }
        """,
        variable_values={"value": value},
    )


@pytest.mark.parametrize(
    "value",
    [
        "2012-12-01T09:00",
        "2012-13-01",
        "2012-04-9",
        #  this might have been fixed in 3.11
        # "20120411",
    ],
)
def test_serialization_of_incorrect_date_string(value):
    """Test GraphQLError is raised for incorrect date.
    The error should exclude "original_error".
    """
    result = execute_mutation(value)
    assert result.errors
    assert isinstance(result.errors[0], GraphQLError)


def test_serialization_error_message_for_incorrect_date_string():
    """Test if error message is using original error message from
    date lib, and is properly formatted.
    """
    result = execute_mutation("2021-13-01")
    assert result.errors
    assert result.errors[0].message.startswith(
        "Variable '$value' got invalid value '2021-13-01'; Value cannot represent a "
        'Date: "2021-13-01". month must be in 1..12'
    )



import tempfile
import time
from pathlib import Path

import strawberry
from strawberry.extensions import pyinstrument


def function_called_by_us_a():
    time.sleep(0.1)
    return function_called_by_us_b()


def function_called_by_us_b():
    time.sleep(0.1)
    return function_called_by_us_c()


def function_called_by_us_c():
    time.sleep(0.1)
    return 4


def test_basic_pyinstrument():
    with tempfile.NamedTemporaryFile(delete=False) as report_file:
        report_file_path = Path(report_file.name)

        @strawberry.type
        class Query:
            @strawberry.field
            def the_field(self) -> int:
                return function_called_by_us_a()

        schema = strawberry.Schema(
            query=Query,
            extensions=[pyinstrument.PyInstrument(report_path=report_file_path)],
        )

        # Query the schema
        result = schema.execute_sync("{ theField }")
        content = report_file_path.read_text("utf-8")

    assert not result.errors
    assert result.data
    assert result.data["theField"] == 4

    assert "function_called_by_us_a" in content
    assert "function_called_by_us_b" in content
    assert "function_called_by_us_c" in content

    assert content.count('"sleep') == 3



import contextlib
import json
from typing import Any
from unittest.mock import patch

import pytest
from graphql import ExecutionResult as GraphQLExecutionResult
from graphql import GraphQLError
from graphql import execute as original_execute

import strawberry
from strawberry.exceptions import StrawberryGraphQLError
from strawberry.extensions import SchemaExtension

from .conftest import ExampleExtension, ExecType, SchemaHelper, hook_wrap


def test_base_extension():
    @strawberry.type
    class Person:
        name: str = "Jess"

    @strawberry.type
    class Query:
        @strawberry.field
        def person(self) -> Person:
            return Person()

    schema = strawberry.Schema(query=Query, extensions=[SchemaExtension])

    query = """
        query {
            person {
                name
            }
        }
    """

    result = schema.execute_sync(query)

    assert not result.errors

    assert result.extensions == {}


def test_called_only_if_overriden(monkeypatch: pytest.MonkeyPatch):
    called = False

    def dont_call_me(self_):
        nonlocal called
        called = True

    class ExtensionNoHooks(SchemaExtension): ...

    for hook in (
        ExtensionNoHooks.on_parse,
        ExtensionNoHooks.on_operation,
        ExtensionNoHooks.on_execute,
        ExtensionNoHooks.on_validate,
    ):
        monkeypatch.setattr(SchemaExtension, hook.__name__, dont_call_me)

    @strawberry.type
    class Person:
        name: str = "Jess"

    @strawberry.type
    class Query:
        @strawberry.field
        def person(self) -> Person:
            return Person()

    schema = strawberry.Schema(query=Query, extensions=[ExtensionNoHooks])

    query = """
        query {
            person {
                name
            }
        }
    """

    result = schema.execute_sync(query)

    assert not result.errors

    assert result.extensions == {}
    assert not called


def test_extension_access_to_parsed_document():
    query_name = ""

    class MyExtension(SchemaExtension):
        def on_parse(self):
            nonlocal query_name
            yield
            query_definition = self.execution_context.graphql_document.definitions[0]
            query_name = query_definition.name.value

    @strawberry.type
    class Person:
        name: str = "Jess"

    @strawberry.type
    class Query:
        @strawberry.field
        def person(self) -> Person:
            return Person()

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    query = """
        query TestQuery {
            person {
                name
            }
        }
    """

    result = schema.execute_sync(query)

    assert not result.errors
    assert query_name == "TestQuery"


def test_extension_access_to_errors():
    execution_errors = []

    class MyExtension(SchemaExtension):
        def on_operation(self):
            nonlocal execution_errors
            yield
            execution_errors = self.execution_context.pre_execution_errors

    @strawberry.type
    class Person:
        name: str = "Jess"

    @strawberry.type
    class Query:
        @strawberry.field
        def person(self) -> Person:
            return None  # type: ignore

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    query = """
        query TestQuery {
            person {
                name
            }
        }
    """

    result = schema.execute_sync(query)

    assert len(result.errors) == 1
    assert execution_errors == result.errors


def test_extension_access_to_root_value():
    root_value = None

    class MyExtension(SchemaExtension):
        def on_operation(self):
            nonlocal root_value
            yield
            root_value = self.execution_context.root_value

    @strawberry.type
    class Query:
        @strawberry.field
        def hi(self) -> str:
            return "👋"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    query = "{ hi }"

    result = schema.execute_sync(query, root_value="ROOT")

    assert not result.errors
    assert root_value == "ROOT"


def test_extension_access_to_operation_extensions():
    operation_extensions = None

    class MyExtension(SchemaExtension):
        def on_operation(self):
            nonlocal operation_extensions
            yield
            operation_extensions = self.execution_context.operation_extensions

    @strawberry.type
    class Query:
        @strawberry.field
        def hi(self) -> str:
            return "👋"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    query = "{ hi }"

    result = schema.execute_sync(query, operation_extensions={"MyExtension": {}})

    assert not result.errors
    assert operation_extensions == {"MyExtension": {}}


def test_can_initialize_extension(default_query_types_and_query):
    class CustomizableExtension(SchemaExtension):
        def __init__(self, arg: int):
            self.arg = arg

        def on_operation(self):
            yield
            self.execution_context.result.data = {"override": self.arg}

    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type,
        extensions=[
            CustomizableExtension(20),
        ],
    )
    res = schema.execute_sync(query=default_query_types_and_query.query)
    assert not res.errors
    assert res.data == {"override": 20}


@pytest.fixture
def sync_extension() -> type[ExampleExtension]:
    class MyExtension(ExampleExtension):
        def on_operation(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_operation.__name__):
                yield

        def on_validate(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_validate.__name__):
                yield

        def on_parse(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_parse.__name__):
                yield

        def on_execute(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_execute.__name__):
                yield

        def get_results(self):
            self.called_hooks.append("get_results")
            return {"example": "example"}

        def resolve(self, _next, root, info, *args: str, **kwargs: Any):
            self.called_hooks.append("resolve")
            return _next(root, info, *args, **kwargs)

    return MyExtension


@pytest.mark.asyncio
async def test_async_extension_hooks(
    default_query_types_and_query: SchemaHelper, async_extension: type[ExampleExtension]
):
    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type, extensions=[async_extension]
    )

    result = await schema.execute(default_query_types_and_query.query)
    assert result.errors is None

    async_extension.assert_expected()


@pytest.mark.asyncio
async def test_mixed_sync_and_async_extension_hooks(
    default_query_types_and_query, sync_extension
):
    class MyExtension(sync_extension):
        async def on_operation(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_operation.__name__):
                yield

        async def on_parse(self):
            with hook_wrap(self.called_hooks, SchemaExtension.on_parse.__name__):
                yield

    @strawberry.type
    class Person:
        name: str = "Jess"

    @strawberry.type
    class Query:
        @strawberry.field
        def person(self) -> Person:
            return Person()

    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type, extensions=[MyExtension]
    )
    result = await schema.execute(default_query_types_and_query.query)
    assert result.errors is None
    MyExtension.assert_expected()


async def test_execution_order(default_query_types_and_query):
    called_hooks = []

    @contextlib.contextmanager
    def register_hook(hook_name: str, klass: type):
        called_hooks.append(f"{klass.__name__}, {hook_name} Entered")
        yield
        called_hooks.append(f"{klass.__name__}, {hook_name} Exited")

    class ExtensionA(ExampleExtension):
        async def on_operation(self):
            with register_hook(SchemaExtension.on_operation.__name__, ExtensionA):
                yield

        async def on_parse(self):
            with register_hook(SchemaExtension.on_parse.__name__, ExtensionA):
                yield

        def on_validate(self):
            with register_hook(SchemaExtension.on_validate.__name__, ExtensionA):
                yield

        def on_execute(self):
            with register_hook(SchemaExtension.on_execute.__name__, ExtensionA):
                yield

    class ExtensionB(ExampleExtension):
        async def on_operation(self):
            with register_hook(SchemaExtension.on_operation.__name__, ExtensionB):
                yield

        def on_parse(self):
            with register_hook(SchemaExtension.on_parse.__name__, ExtensionB):
                yield

        def on_validate(self):
            with register_hook(SchemaExtension.on_validate.__name__, ExtensionB):
                yield

        async def on_execute(self):
            with register_hook(SchemaExtension.on_execute.__name__, ExtensionB):
                yield

    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type,
        extensions=[ExtensionA, ExtensionB],
    )
    result = await schema.execute(default_query_types_and_query.query)
    assert result.errors is None
    assert called_hooks == [
        "ExtensionA, on_operation Entered",
        "ExtensionB, on_operation Entered",
        "ExtensionA, on_parse Entered",
        "ExtensionB, on_parse Entered",
        "ExtensionB, on_parse Exited",
        "ExtensionA, on_parse Exited",
        "ExtensionA, on_validate Entered",
        "ExtensionB, on_validate Entered",
        "ExtensionB, on_validate Exited",
        "ExtensionA, on_validate Exited",
        "ExtensionA, on_execute Entered",
        "ExtensionB, on_execute Entered",
        "ExtensionB, on_execute Exited",
        "ExtensionA, on_execute Exited",
        "ExtensionB, on_operation Exited",
        "ExtensionA, on_operation Exited",
    ]


async def test_sync_extension_hooks(default_query_types_and_query, sync_extension):
    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type,
        extensions=[
            sync_extension,
        ],
    )

    result = schema.execute_sync(default_query_types_and_query.query)
    assert result.errors is None

    sync_extension.assert_expected()


async def test_extension_no_yield(default_query_types_and_query):
    class SyncExt(ExampleExtension):
        expected = [
            f"{SchemaExtension.on_operation.__name__} Entered",
            f"{SchemaExtension.on_parse.__name__} Entered",
        ]

        def on_operation(self):
            self.called_hooks.append(self.__class__.expected[0])

        async def on_parse(self):
            self.called_hooks.append(self.__class__.expected[1])

    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type, extensions=[SyncExt]
    )

    result = await schema.execute(default_query_types_and_query.query)
    assert result.errors is None

    SyncExt.assert_expected()


def test_warning_about_async_get_results_hooks_in_sync_context():
    class MyExtension(SchemaExtension):
        async def get_results(self):
            pass

    @strawberry.type
    class Query:
        @strawberry.field
        def string(self) -> str:
            return ""

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])
    query = "query { string }"

    with pytest.raises(
        RuntimeError, match="Cannot use async extension hook during sync execution"
    ):
        schema.execute_sync(query)


class ExceptionTestingExtension(SchemaExtension):
    def __init__(self, failing_hook: str):
        self.failing_hook = failing_hook
        self.called_hooks = set()

    def on_operation(self):
        if self.failing_hook == "on_operation_start":
            raise Exception(self.failing_hook)
        self.called_hooks.add(1)

        with contextlib.suppress(Exception):
            yield

        if self.failing_hook == "on_operation_end":
            raise Exception(self.failing_hook)
        self.called_hooks.add(8)

    def on_parse(self):
        if self.failing_hook == "on_parse_start":
            raise Exception(self.failing_hook)
        self.called_hooks.add(2)

        with contextlib.suppress(Exception):
            yield

        if self.failing_hook == "on_parse_end":
            raise Exception(self.failing_hook)
        self.called_hooks.add(3)

    def on_validate(self):
        if self.failing_hook == "on_validate_start":
            raise Exception(self.failing_hook)
        self.called_hooks.add(4)

        with contextlib.suppress(Exception):
            yield

        if self.failing_hook == "on_validate_end":
            raise Exception(self.failing_hook)
        self.called_hooks.add(5)

    def on_execute(self):
        if self.failing_hook == "on_execute_start":
            raise Exception(self.failing_hook)
        self.called_hooks.add(6)

        with contextlib.suppress(Exception):
            yield

        if self.failing_hook == "on_execute_end":
            raise Exception(self.failing_hook)
        self.called_hooks.add(7)


@pytest.mark.parametrize(
    "failing_hook",
    [
        "on_operation_start",
        "on_operation_end",
        "on_parse_start",
        "on_parse_end",
        "on_validate_start",
        "on_validate_end",
        "on_execute_start",
        "on_execute_end",
    ],
)
@pytest.mark.asyncio
async def test_exceptions_are_included_in_the_execution_result(failing_hook):
    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(
        query=Query,
        extensions=[ExceptionTestingExtension(failing_hook)],
    )
    document = "query { ping }"

    sync_result = schema.execute_sync(document)
    assert sync_result.errors is not None
    assert len(sync_result.errors) == 1
    assert sync_result.errors[0].message == failing_hook

    async_result = await schema.execute(document)
    assert async_result.errors is not None
    assert len(async_result.errors) == 1
    assert sync_result.errors[0].message == failing_hook


@pytest.mark.parametrize(
    ("failing_hook", "expected_hooks"),
    [
        ("on_operation_start", set()),
        ("on_parse_start", {1, 8}),
        ("on_parse_end", {1, 2, 8}),
        ("on_validate_start", {1, 2, 3, 8}),
        ("on_validate_end", {1, 2, 3, 4, 8}),
        ("on_execute_start", {1, 2, 3, 4, 5, 8}),
        ("on_execute_end", {1, 2, 3, 4, 5, 6, 8}),
        ("on_operation_end", {1, 2, 3, 4, 5, 6, 7}),
    ],
)
@pytest.mark.asyncio
async def test_exceptions_abort_evaluation(failing_hook, expected_hooks):
    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    extension = ExceptionTestingExtension(failing_hook)
    schema = strawberry.Schema(query=Query, extensions=[extension])
    document = "query { ping }"

    extension.called_hooks = set()
    schema.execute_sync(document)
    assert extension.called_hooks == expected_hooks

    extension.called_hooks = set()
    await schema.execute(document)
    assert extension.called_hooks == expected_hooks


async def test_generic_exceptions_get_wrapped_in_a_graphql_error(
    exec_type: ExecType,
) -> None:
    exception = Exception("This should be wrapped in a GraphQL error")

    class MyExtension(SchemaExtension):
        def on_parse(self):
            raise exception

    @strawberry.type
    class Query:
        ping: str = "pong"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])
    query = "query { ping }"
    if exec_type.is_async():
        res = await schema.execute(query)
    else:
        res = schema.execute_sync(query)

    res = await schema.execute(query)
    assert len(res.errors) == 1
    assert isinstance(res.errors[0], GraphQLError)
    assert res.errors[0].original_error == exception


async def test_graphql_errors_get_not_wrapped_in_a_graphql_error(
    exec_type: ExecType,
) -> None:
    exception = GraphQLError("This should not be wrapped in a GraphQL error")

    class MyExtension(SchemaExtension):
        def on_parse(self):
            raise exception

    @strawberry.type
    class Query:
        ping: str = "pong"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])
    query = "query { ping }"
    if exec_type.is_async():
        res = await schema.execute(query)
    else:
        res = schema.execute_sync(query)
    assert len(res.errors) == 1
    assert res.errors[0] == exception
    assert res.errors[0].original_error is None


@pytest.mark.asyncio
async def test_dont_swallow_errors_in_parsing_hooks():
    class MyExtension(SchemaExtension):
        def on_parse(self):
            raise Exception("This shouldn't be swallowed")

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])
    query = "query { string }"

    sync_result = schema.execute_sync(query)
    assert len(sync_result.errors) == 1
    assert sync_result.errors[0].message == "This shouldn't be swallowed"

    async_result = await schema.execute(query)
    assert len(async_result.errors) == 1
    assert async_result.errors[0].message == "This shouldn't be swallowed"


def test_on_parsing_end_is_called_with_parsing_errors():
    execution_errors = False

    class MyExtension(SchemaExtension):
        def on_parse(self):
            nonlocal execution_errors
            yield
            execution_context = self.execution_context
            execution_errors = execution_context.pre_execution_errors

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])
    query = "query { string"  # Invalid query

    result = schema.execute_sync(query)
    assert result.errors

    assert result.errors == execution_errors


def test_extension_execution_order_sync():
    """Ensure mixed hooks (async & sync) are called correctly."""
    execution_order: list[type[SchemaExtension]] = []

    class ExtensionB(SchemaExtension):
        def on_execute(self):
            execution_order.append(type(self))
            yield
            execution_order.append(type(self))

    class ExtensionC(SchemaExtension):
        def on_execute(self):
            execution_order.append(type(self))
            yield
            execution_order.append(type(self))

    @strawberry.type
    class Query:
        food: str = "strawberry"

    extensions = [ExtensionB, ExtensionC]
    schema = strawberry.Schema(query=Query, extensions=extensions)

    query = """
        query TestQuery {
            food
        }
    """

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors
    assert result.data == {"food": "strawberry"}
    assert execution_order == [ExtensionB, ExtensionC, ExtensionC, ExtensionB]


def test_async_extension_in_sync_context():
    class ExtensionA(SchemaExtension):
        async def on_execute(self):
            yield

    @strawberry.type
    class Query:
        food: str = "strawberry"

    schema = strawberry.Schema(query=Query, extensions=[ExtensionA])

    result = schema.execute_sync("query { food }")
    assert len(result.errors) == 1
    assert result.errors[0].message.endswith("failed to complete synchronously.")


def test_extension_override_execution():
    class MyExtension(SchemaExtension):
        def on_execute(self):
            # Always return a static response
            self.execution_context.result = GraphQLExecutionResult(
                data={
                    "surprise": "data",
                },
                errors=[],
            )

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    query = """
        query TestQuery {
            ping
        }
    """

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {
        "surprise": "data",
    }


@pytest.mark.asyncio
async def test_extension_override_execution_async():
    class MyExtension(SchemaExtension):
        def on_execute(self):
            # Always return a static response
            self.execution_context.result = GraphQLExecutionResult(
                data={
                    "surprise": "data",
                },
                errors=[],
            )

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    query = """
        query TestQuery {
            ping
        }
    """

    result = await schema.execute(query)

    assert not result.errors
    assert result.data == {
        "surprise": "data",
    }


@patch("strawberry.schema.schema.execute", wraps=original_execute)
def test_execution_cache_example(mock_original_execute):
    # Test that the example of how to use the on_execute hook in the
    # docs actually works

    response_cache = {}

    class ExecutionCache(SchemaExtension):
        def on_execute(self):
            # Check if we've come across this query before
            execution_context = self.execution_context
            self.cache_key = (
                f"{execution_context.query}:{json.dumps(execution_context.variables)}"
            )
            if self.cache_key in response_cache:
                self.execution_context.result = response_cache[self.cache_key]
            yield
            if self.cache_key not in response_cache:
                response_cache[self.cache_key] = execution_context.result

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self, return_value: str | None = None) -> str:
            if return_value is not None:
                return return_value
            return "pong"

    schema = strawberry.Schema(
        Query,
        extensions=[
            ExecutionCache,
        ],
    )

    query = """
        query TestQuery($returnValue: String) {
            ping(returnValue: $returnValue)
        }
    """
    result = schema.execute_sync(query)
    assert not result.errors
    assert result.data == {
        "ping": "pong",
    }

    assert mock_original_execute.call_count == 1

    # This should be cached
    result = schema.execute_sync(query)
    assert not result.errors
    assert result.data == {
        "ping": "pong",
    }

    assert mock_original_execute.call_count == 1

    # Calling with different variables should not be cached
    result = schema.execute_sync(
        query,
        variable_values={
            "returnValue": "plong",
        },
    )
    assert not result.errors
    assert result.data == {
        "ping": "plong",
    }

    assert mock_original_execute.call_count == 2


@patch("strawberry.schema.schema.execute", wraps=original_execute)
def test_execution_reject_example(mock_original_execute):
    # Test that the example of how to use the on_execute hook in the
    # docs actually works

    class RejectSomeQueries(SchemaExtension):
        def on_execute(self):
            # Reject all operations called "RejectMe"
            execution_context = self.execution_context
            if execution_context.operation_name == "RejectMe":
                self.execution_context.result = GraphQLExecutionResult(
                    data=None,
                    errors=[GraphQLError("Well you asked for it")],
                )

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(
        Query,
        extensions=[
            RejectSomeQueries,
        ],
    )

    query = """
        query TestQuery {
            ping
        }
    """
    result = schema.execute_sync(query, operation_name="TestQuery")
    assert not result.errors
    assert result.data == {
        "ping": "pong",
    }

    assert mock_original_execute.call_count == 1

    query = """
        query RejectMe {
            ping
        }
    """
    result = schema.execute_sync(query, operation_name="RejectMe")
    assert result.errors == [GraphQLError("Well you asked for it")]

    assert mock_original_execute.call_count == 1


def test_extend_error_format_example():
    # Test that the example of how to extend error format

    class ExtendErrorFormat(SchemaExtension):
        def on_operation(self):
            yield
            result = self.execution_context.result
            if getattr(result, "errors", None):
                result.errors = [
                    StrawberryGraphQLError(
                        extensions={"additional_key": "additional_value"},
                        nodes=error.nodes,
                        source=error.source,
                        positions=error.positions,
                        path=error.path,
                        original_error=error.original_error,
                        message=error.message,
                    )
                    for error in result.errors
                ]

    @strawberry.type
    class Query:
        @strawberry.field
        def ping(self) -> str:
            raise Exception("This error occurred while querying the ping field")

    schema = strawberry.Schema(query=Query, extensions=[ExtendErrorFormat])
    query = """
        query TestQuery {
            ping
        }
    """

    result = schema.execute_sync(query)
    assert result.errors[0].extensions == {"additional_key": "additional_value"}
    assert (
        result.errors[0].message == "This error occurred while querying the ping field"
    )
    assert result.data is None


def test_extension_can_set_query():
    class MyExtension(SchemaExtension):
        def on_operation(self):
            self.execution_context.query = "{ hi }"
            yield

    @strawberry.type
    class Query:
        @strawberry.field
        def hi(self) -> str:
            return "👋"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    # Query not set on input
    query = ""

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"hi": "👋"}


@pytest.mark.asyncio
async def test_extension_can_set_query_async():
    class MyExtension(SchemaExtension):
        def on_operation(self):
            self.execution_context.query = "{ hi }"
            yield

    @strawberry.type
    class Query:
        @strawberry.field
        async def hi(self) -> str:
            return "👋"

    schema = strawberry.Schema(query=Query, extensions=[MyExtension])

    # Query not set on input
    query = ""

    result = await schema.execute(query)

    assert not result.errors
    assert result.data == {"hi": "👋"}


def test_raise_if_hook_is_not_callable(default_query_types_and_query: SchemaHelper):
    class MyExtension(SchemaExtension):
        on_operation = "ABC"  # type: ignore

    schema = strawberry.Schema(
        query=default_query_types_and_query.query_type, extensions=[MyExtension]
    )
    result = schema.execute_sync(default_query_types_and_query.query)
    assert len(result.errors) == 1
    assert isinstance(result.errors[0].original_error, ValueError)
    assert result.errors[0].message.startswith("Hook on_operation on <")
    assert result.errors[0].message.endswith("> must be callable, received 'ABC'")



import contextlib

import pytest

from tests.http.clients.base import HttpClient
from tests.views.schema import schema


@pytest.fixture
def incremental_http_client_class(
    http_client_class: type[HttpClient],
) -> type[HttpClient]:
    with contextlib.suppress(ImportError):
        import django

        if django.VERSION < (4, 2):
            pytest.skip(reason="Django < 4.2 doesn't async streaming responses")

        from tests.http.clients.django import DjangoHttpClient

        if http_client_class is DjangoHttpClient:
            pytest.skip(reason="(sync) DjangoHttpClient doesn't support streaming")

    with contextlib.suppress(ImportError):
        from tests.http.clients.channels import SyncChannelsHttpClient

        # TODO: why do we have a sync channels client?
        if http_client_class is SyncChannelsHttpClient:
            pytest.skip(reason="SyncChannelsHttpClient doesn't support streaming")

    with contextlib.suppress(ImportError):
        from tests.http.clients.async_flask import AsyncFlaskHttpClient
        from tests.http.clients.flask import FlaskHttpClient

        if http_client_class is FlaskHttpClient:
            pytest.skip(reason="FlaskHttpClient doesn't support streaming")

        if http_client_class is AsyncFlaskHttpClient:
            pytest.xfail(reason="AsyncFlaskHttpClient doesn't support streaming")

    with contextlib.suppress(ImportError):
        from tests.http.clients.chalice import ChaliceHttpClient

        if http_client_class is ChaliceHttpClient:
            pytest.skip(reason="ChaliceHttpClient doesn't support streaming")

    return http_client_class


@pytest.fixture
def http_client(incremental_http_client_class: type[HttpClient]) -> HttpClient:
    return incremental_http_client_class(schema)



import pytest


def test_can_import(mocker):
    # mocking sys.modules.ddtrace so we don't get an ImportError
    mocker.patch.dict("sys.modules", ddtrace=mocker.MagicMock())


def test_fails_if_import_is_not_found():
    with pytest.raises(ImportError):
        from strawberry.extensions.tracing import Blueberry  # noqa: F401



from unittest.mock import patch

import pytest
from graphql import validate

import strawberry
from strawberry.extensions import ValidationCache


@patch("strawberry.schema.schema.validate", wraps=validate)
def test_validation_cache_extension(mock_validate):
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[ValidationCache()])

    query = "query { hello }"

    result = schema.execute_sync(query)

    assert not result.errors
    assert result.data == {"hello": "world"}

    assert mock_validate.call_count == 1

    # Run query multiple times
    for _ in range(3):
        result = schema.execute_sync(query)
        assert not result.errors
        assert result.data == {"hello": "world"}

    # validate is still only called once
    assert mock_validate.call_count == 1

    # Running a second query doesn't cache
    query2 = "query { ping }"
    result = schema.execute_sync(query2)

    assert not result.errors
    assert result.data == {"ping": "pong"}

    assert mock_validate.call_count == 2


@patch("strawberry.schema.schema.validate", wraps=validate)
def test_validation_cache_extension_max_size(mock_validate):
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[ValidationCache(maxsize=1)])

    query = "query { hello }"

    for _ in range(2):
        result = schema.execute_sync(query)
        assert not result.errors
        assert result.data == {"hello": "world"}

    assert mock_validate.call_count == 1

    query2 = "query { ping }"
    result = schema.execute_sync(query2)

    assert not result.errors
    assert mock_validate.call_count == 2

    # running the first query again doesn't cache
    result = schema.execute_sync(query)
    assert not result.errors

    # validate is still only called once
    assert mock_validate.call_count == 3


@pytest.mark.asyncio
async def test_validation_cache_extension_async():
    @strawberry.type
    class Query:
        @strawberry.field
        def hello(self) -> str:
            return "world"

        @strawberry.field
        def ping(self) -> str:
            return "pong"

    schema = strawberry.Schema(query=Query, extensions=[ValidationCache()])

    query = "query { hello }"

    with patch("strawberry.schema.schema.validate", wraps=validate) as mock_validate:
        result = await schema.execute(query)

        assert not result.errors
        assert result.data == {"hello": "world"}

        assert mock_validate.call_count == 1

        # Run query multiple times
        for _ in range(3):
            result = await schema.execute(query)
            assert not result.errors
            assert result.data == {"hello": "world"}

        # validate is still only called once
        assert mock_validate.call_count == 1

        # Running a second query doesn't cache
        query2 = "query { ping }"
        result = await schema.execute(query2)

        assert not result.errors
        assert result.data == {"ping": "pong"}

        assert mock_validate.call_count == 2



import strawberry

from . import a_mod, b_mod, x_mod
from .a_mod import AObject as C_AObject
from .b_mod import BObject as C_BObject


def c_inheritance_resolver() -> list["CInheritance"]:
    pass


def c_composition_resolver() -> list["CComposition"]:
    pass


def c_composition_by_name_resolver() -> list["CCompositionByName"]:
    pass


@strawberry.type
class CInheritance(a_mod.AObject, b_mod.BObject):
    pass


@strawberry.type
class CComposition:
    a_list: list[a_mod.AObject]
    b_list: list[b_mod.BObject]


@strawberry.type
class CCompositionByName:
    a_list: list["C_AObject"]
    b_list: list["C_BObject"]

    @strawberry.field
    def a_method(self) -> list["C_AObject"]:
        return self.a_list

    @strawberry.field
    def b_method(self) -> list["C_BObject"]:
        return self.b_list


@strawberry.type
class CCompositionByNameWithResolvers:
    a_list: list["C_AObject"] = strawberry.field(resolver=a_mod.a_resolver)
    b_list: list["C_BObject"] = strawberry.field(resolver=b_mod.b_resolver)


@strawberry.type
class CCompositionByNameWithTypelessResolvers:
    a_list: list["C_AObject"] = strawberry.field(resolver=x_mod.typeless_resolver)
    b_list: list["C_BObject"] = strawberry.field(resolver=x_mod.typeless_resolver)


@strawberry.type
class CCompositionOnlyResolvers:
    a_list = strawberry.field(resolver=a_mod.a_resolver)
    b_list = strawberry.field(resolver=b_mod.b_resolver)



import textwrap
from enum import Enum
from typing import Annotated, Generic, TypeVar

import strawberry
from strawberry.directive import StrawberryDirective
from strawberry.schema.config import StrawberryConfig
from strawberry.schema.name_converter import NameConverter
from strawberry.schema_directive import Location, StrawberrySchemaDirective
from strawberry.types.arguments import StrawberryArgument
from strawberry.types.base import StrawberryObjectDefinition, StrawberryType
from strawberry.types.enum import EnumValue, StrawberryEnumDefinition
from strawberry.types.field import StrawberryField
from strawberry.types.scalar import ScalarDefinition
from strawberry.types.union import StrawberryUnion


class AppendsNameConverter(NameConverter):
    def __init__(self, suffix: str):
        self.suffix = suffix
        super().__init__(auto_camel_case=True)

    def from_argument(self, argument: StrawberryArgument) -> str:
        return super().from_argument(argument) + self.suffix

    def from_scalar(self, scalar: ScalarDefinition) -> str:
        return super().from_scalar(scalar) + self.suffix

    def from_field(self, field: StrawberryField) -> str:
        return super().from_field(field) + self.suffix

    def from_union(self, union: StrawberryUnion) -> str:
        return super().from_union(union) + self.suffix

    def from_generic(
        self,
        generic_type: StrawberryObjectDefinition,
        types: list[StrawberryType | type],
    ) -> str:
        return super().from_generic(generic_type, types) + self.suffix

    def from_interface(self, interface: StrawberryObjectDefinition) -> str:
        return super().from_interface(interface) + self.suffix

    def from_directive(
        self, directive: StrawberryDirective | StrawberrySchemaDirective
    ) -> str:
        return super().from_directive(directive) + self.suffix

    def from_input_object(self, input_type: StrawberryObjectDefinition) -> str:
        return super().from_object(input_type) + self.suffix

    def from_object(self, object_type: StrawberryObjectDefinition) -> str:
        return super().from_object(object_type) + self.suffix

    def from_enum(self, enum: StrawberryEnumDefinition) -> str:
        return super().from_enum(enum) + self.suffix

    def from_enum_value(
        self, enum: StrawberryEnumDefinition, enum_value: EnumValue
    ) -> str:
        return super().from_enum_value(enum, enum_value) + self.suffix


# TODO: maybe we should have a kitchen sink schema that we can reuse for tests

T = TypeVar("T")

MyScalar = strawberry.scalar(str, name="SensitiveConfiguration")


@strawberry.enum
class MyEnum(Enum):
    A = "a"
    B = "b"


@strawberry.type
class User:
    name: str


@strawberry.type(name="MyType")
class TypeWithDifferentNameThanClass:
    name: str


@strawberry.type
class Error:
    message: str


@strawberry.input
class UserInput:
    name: str


@strawberry.schema_directive(
    locations=[
        Location.FIELD_DEFINITION,
    ]
)
class MyDirective:
    name: str


@strawberry.interface
class Node:
    id: strawberry.ID


@strawberry.type
class MyGeneric(Generic[T]):
    value: T


@strawberry.type
class Query:
    @strawberry.field(directives=[MyDirective(name="my-directive")])
    def user(self, input: UserInput) -> User | Error:
        return User(name="Patrick")

    enum: MyEnum = MyEnum.A
    field: MyGeneric[str] | None = None
    field_with_lazy: MyGeneric[
        Annotated[
            "TypeWithDifferentNameThanClass",
            strawberry.lazy("tests.schema.test_name_converter"),
        ]
    ] = None

    @strawberry.field
    def print(self, enum: MyEnum) -> str:
        return enum.value


schema = strawberry.Schema(
    query=Query,
    types=[MyScalar, Node],
    config=StrawberryConfig(name_converter=AppendsNameConverter("X")),
)


def test_name_converter():
    expected_schema = """
    directive @myDirectiveX(name: String!) on FIELD_DEFINITION

    type ErrorX {
      messageX: String!
    }

    enum MyEnumX {
      AX
      BX
    }

    type MyTypeMyGenericXX {
      valueX: MyTypeX!
    }

    type MyTypeX {
      nameX: String!
    }

    interface NodeXX {
      idX: ID!
    }

    type QueryX {
      enumX: MyEnumX!
      fieldX: StrMyGenericXX
      fieldWithLazyX: MyTypeMyGenericXX!
      userX(inputX: UserInputX!): UserXErrorXX! @myDirectiveX(name: "my-directive")
      printX(enumX: MyEnumX!): String!
    }

    scalar SensitiveConfiguration

    type StrMyGenericXX {
      valueX: String!
    }

    input UserInputX {
      nameX: String!
    }

    type UserX {
      nameX: String!
    }

    union UserXErrorXX = UserX | ErrorX
    """

    assert textwrap.dedent(expected_schema).strip() == str(schema)


def test_returns_enum_with_correct_value():
    query = " { enumX } "

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors

    assert result.data == {"enumX": "AX"}


def test_can_use_enum_value():
    query = " { printX(enumX: AX) } "

    result = schema.execute_sync(query, root_value=Query())

    assert not result.errors

    assert result.data == {"printX": "a"}


def test_can_use_enum_value_with_variable():
    query = " query ($enum: MyEnumX!) { printX(enumX: $enum) } "

    result = schema.execute_sync(
        query, root_value=Query(), variable_values={"enum": "AX"}
    )

    assert not result.errors

    assert result.data == {"printX": "a"}



"""Benchmark for a complex nested query with a large stadium dataset.

This benchmark tests Strawberry's performance when dealing with deeply nested
objects and large result sets. The stadium query generates approximately 50,000
seat objects across multiple stands, each with multiple labels and coordinates.
"""

import asyncio
from pathlib import Path

import pytest
from pytest_codspeed.plugin import BenchmarkFixture

import strawberry


@strawberry.type
class Seat:
    x: int
    y: int
    labels: list[str]


@strawberry.type
class Stand:
    name: str
    seats: list[Seat]
    section_type: str
    price_category: str


@strawberry.type
class Stadium:
    name: str
    city: str
    country: str
    stands: list[Stand]


def generate_seats_for_stand(
    stand_name: str,
    rows: int,
    seats_per_row: int,
    x_offset: int,
    y_offset: int,
    section_type: str,
) -> list[Seat]:
    """Generate seats for a stand with proper coordinates and labels."""
    seats = []

    for row in range(rows):
        for seat_num in range(seats_per_row):
            x = x_offset + seat_num
            y = y_offset + row

            row_label = (
                chr(65 + row)
                if row < 26
                else f"{chr(65 + row // 26)}{chr(65 + row % 26)}"
            )

            labels = [
                stand_name,
                f"Row-{row_label}",
                f"Seat-{seat_num + 1}",
                section_type,
                f"Block-{(row // 5) + 1}",
            ]

            seats.append(Seat(x=x, y=y, labels=labels))

    return seats


def create_stadium(seats_per_row: int = 250) -> Stadium:
    """Create a stadium with a configurable number of seats per row.

    Default configuration (250 seats/row) creates approximately 50,000 seats:
    - North Stand: 12,500 seats (50 rows × 250 seats)
    - South Stand: 12,500 seats (50 rows × 250 seats)
    - East Stand: 10,000 seats (40 rows × 250 seats)
    - West Stand: 10,000 seats (40 rows × 250 seats)
    """
    stands = []

    # North Stand
    north_stand_seats = generate_seats_for_stand(
        stand_name="North-Stand",
        rows=50,
        seats_per_row=seats_per_row,
        x_offset=0,
        y_offset=0,
        section_type="Standard",
    )
    stands.append(
        Stand(
            name="North Stand",
            seats=north_stand_seats,
            section_type="Standard",
            price_category="Bronze",
        )
    )

    # South Stand
    south_stand_seats = generate_seats_for_stand(
        stand_name="South-Stand",
        rows=50,
        seats_per_row=seats_per_row,
        x_offset=0,
        y_offset=100,
        section_type="Standard",
    )
    stands.append(
        Stand(
            name="South Stand",
            seats=south_stand_seats,
            section_type="Standard",
            price_category="Bronze",
        )
    )

    # East Stand
    east_stand_seats = generate_seats_for_stand(
        stand_name="East-Stand",
        rows=40,
        seats_per_row=seats_per_row,
        x_offset=300,
        y_offset=20,
        section_type="Premium",
    )
    stands.append(
        Stand(
            name="East Stand",
            seats=east_stand_seats,
            section_type="Premium",
            price_category="Gold",
        )
    )

    # West Stand
    west_stand_seats = generate_seats_for_stand(
        stand_name="West-Stand",
        rows=40,
        seats_per_row=seats_per_row,
        x_offset=-300,
        y_offset=20,
        section_type="Premium",
    )
    stands.append(
        Stand(
            name="West Stand",
            seats=west_stand_seats,
            section_type="Premium",
            price_category="Gold",
        )
    )

    return Stadium(
        name="Grand Metropolitan Stadium",
        city="London",
        country="United Kingdom",
        stands=stands,
    )


@strawberry.type
class Query:
    @strawberry.field
    def stadium(self, seats_per_row: int) -> Stadium:
        return create_stadium(seats_per_row)


ROOT = Path(__file__).parent / "queries"
stadium_query = (ROOT / "stadium.graphql").read_text()


@pytest.mark.benchmark
@pytest.mark.parametrize(
    "seats_per_row", [250, 500], ids=lambda x: f"seats_per_row_{x}"
)
def test_stadium(benchmark: BenchmarkFixture, seats_per_row: int):
    """Benchmark a complex nested query with a large dataset.

    This test benchmarks the execution of a GraphQL query that returns
    a stadium with multiple stands, each containing thousands of seats.
    Each seat has multiple labels and coordinates.

    The benchmark tests with different seat counts:
    - 250 seats/row: ~45,000 total seats
    - 500 seats/row: ~90,000 total seats
    """
    schema = strawberry.Schema(query=Query)

    def run():
        return asyncio.run(
            schema.execute(
                stadium_query, variable_values={"seatsPerRow": seats_per_row}
            )
        )

    results = benchmark(run)

    assert results.errors is None
    assert results.data is not None
    assert results.data["stadium"]["name"] == "Grand Metropolitan Stadium"



import strawberry


def test_can_instantiate_types_directly():
    @strawberry.type
    class User:
        username: str

        @strawberry.field
        def email(self) -> str:
            return self.username + "@somesite.com"

    user = User(username="abc")
    assert user.username == "abc"
    assert user.email() == "abc@somesite.com"



import strawberry


def test_inherited_fields():
    @strawberry.type
    class A:
        a: str = strawberry.field(default="")

    @strawberry.type
    class B(A):
        b: str | None = strawberry.field(default=None)

    assert strawberry.Schema(query=B)



import strawberry
from strawberry.extensions.max_aliases import MaxAliasesLimiter


@strawberry.type
class Human:
    name: str
    email: str


@strawberry.type
class Query:
    @strawberry.field
    def user(self, name: str | None = None, email: str | None = None) -> Human:
        return Human(name="Jane Doe", email="jane@example.com")

    version: str
    user1: Human
    user2: Human
    user3: Human


def test_2_aliases_same_content():
    query = """
    {
      matt: user(name: "matt") {
        email
      }
      matt_alias: user(name: "matt") {
        email
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert len(result.errors) == 1
    assert result.errors[0].message == "2 aliases found. Allowed: 1"


def test_2_aliases_different_content():
    query = """
    query read {
      matt: user(name: "matt") {
        email
      }
      matt_alias: user(name: "matt42") {
        email
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert len(result.errors) == 1
    assert result.errors[0].message == "2 aliases found. Allowed: 1"


def test_multiple_aliases_some_overlap_in_content():
    query = """
    query read {
      matt: user(name: "matt") {
        email
      }
      jane: user(name: "jane") {
        email
      }
      matt_alias: user(name: "matt") {
        email
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert len(result.errors) == 1
    assert result.errors[0].message == "3 aliases found. Allowed: 1"


def test_multiple_arguments():
    query = """
    query read {
      matt: user(name: "matt", email: "matt@example.com") {
        email
      }
      jane: user(name: "jane") {
        email
      }
      matt_alias: user(name: "matt", email: "matt@example.com") {
        email
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert len(result.errors) == 1
    assert result.errors[0].message == "3 aliases found. Allowed: 1"


def test_alias_in_nested_field():
    query = """
    query read {
      matt: user(name: "matt") {
        email_address: email
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert len(result.errors) == 1
    assert result.errors[0].message == "2 aliases found. Allowed: 1"


def test_alias_in_fragment():
    query = """
    fragment humanInfo on Human {
      email_address: email
    }
    query read {
      matt: user(name: "matt") {
        ...humanInfo
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert len(result.errors) == 1
    assert result.errors[0].message == "2 aliases found. Allowed: 1"


def test_2_top_level_1_nested():
    query = """{
      matt: user(name: "matt") {
        email_address: email
      }
      matt_alias: user(name: "matt") {
        email
      }
    }
    """
    result = _execute_with_max_aliases(query, 2)

    assert len(result.errors) == 1
    assert result.errors[0].message == "3 aliases found. Allowed: 2"


def test_no_error_one_aliased_one_without():
    query = """
    {
      user(name: "matt") {
        email
      }
      matt_alias: user(name: "matt") {
        email
      }
    }
    """

    result = _execute_with_max_aliases(query, 1)

    assert not result.errors


def test_no_error_for_multiple_but_not_too_many_aliases():
    query = """{
      matt: user(name: "matt") {
        email
      }
      matt_alias: user(name: "matt") {
        email
      }
    }
    """

    result = _execute_with_max_aliases(query, 2)

    assert not result.errors


def _execute_with_max_aliases(query: str, max_alias_count: int):
    schema = strawberry.Schema(
        Query, extensions=[MaxAliasesLimiter(max_alias_count=max_alias_count)]
    )

    return schema.execute_sync(query)



#!/usr/bin/env python

# we use uv for our build, but this file seems to be required
# in order to get GitHub dependencies graph to work

import setuptools

if __name__ == "__main__":
    setuptools.setup(name="strawberry-graphql")



import typing
from typing import Annotated, ClassVar, ForwardRef, Optional, Union

import strawberry
from strawberry.types.lazy_type import LazyType
from strawberry.utils.typing import eval_type, get_optional_annotation, is_classvar


@strawberry.type
class Fruit: ...


def test_get_optional_annotation():
    # Pair Union
    assert get_optional_annotation(Optional[str | bool]) == Union[str, bool]

    # More than pair Union
    assert get_optional_annotation(Optional[str | int | bool]) == Union[str, int, bool]


def test_eval_type():
    class Foo: ...

    assert eval_type(ForwardRef("str")) is str
    assert eval_type(str) is str
    assert eval_type(ForwardRef("Foo"), None, locals()) is Foo
    assert eval_type(Foo, None, locals()) is Foo
    assert eval_type(ForwardRef("Optional[Foo]"), globals(), locals()) == Optional[Foo]
    assert eval_type(Optional["Foo"], globals(), locals()) == Optional[Foo]
    assert (
        eval_type(ForwardRef("Union[Foo, str]"), globals(), locals()) == Union[Foo, str]
    )
    assert eval_type(Union["Foo", "str"], globals(), locals()) == Union[Foo, str]
    assert (
        eval_type(ForwardRef("Optional[Union[Foo, str]]"), globals(), locals())
        == Union[Foo, str, None]
    )
    assert (
        eval_type(ForwardRef("Annotated[str, 'foobar']"), globals(), locals())
        is Annotated[str, "foobar"]
    )
    assert (
        eval_type(
            ForwardRef("Annotated[Fruit, strawberry.lazy('tests.utils.test_typing')]"),
            {"strawberry": strawberry, "Annotated": Annotated},
            None,
        )
        == Annotated[
            LazyType("Fruit", "tests.utils.test_typing"),
            strawberry.lazy("tests.utils.test_typing"),
        ]
    )
    assert (
        eval_type(
            ForwardRef("Annotated[strawberry.auto, 'foobar']"),
            {"strawberry": strawberry, "Annotated": Annotated},
            None,
        )
        == Annotated[strawberry.auto, "foobar"]
    )
    assert (
        eval_type(
            ForwardRef("Annotated[datetime, strawberry.lazy('datetime')]"),
            {"strawberry": strawberry, "Annotated": Annotated},
            None,
        )
        == Annotated[
            LazyType("datetime", "datetime"),
            strawberry.lazy("datetime"),
        ]
    )


def test_eval_type_with_deferred_annotations():
    assert (
        eval_type(
            ForwardRef(
                "Annotated['Fruit', strawberry.lazy('tests.utils.test_typing')]"
            ),
            {"strawberry": strawberry, "Annotated": Annotated},
            None,
        )
        == Annotated[
            LazyType("Fruit", "tests.utils.test_typing"),
            strawberry.lazy("tests.utils.test_typing"),
        ]
    )
    assert (
        eval_type(
            ForwardRef("Annotated['datetime', strawberry.lazy('datetime')]"),
            {"strawberry": strawberry, "Annotated": Annotated},
            None,
        )
        == Annotated[
            LazyType("datetime", "datetime"),
            strawberry.lazy("datetime"),
        ]
    )


def test_is_classvar():
    class Foo:
        attr1: str
        attr2: ClassVar[str]
        attr3: typing.ClassVar[str]

    assert not is_classvar(Foo, Foo.__annotations__["attr1"])
    assert is_classvar(Foo, Foo.__annotations__["attr2"])
    assert is_classvar(Foo, Foo.__annotations__["attr3"])



from enum import Enum

import strawberry
from strawberry import asdict


def test_convert_simple_type_to_dictionary():
    @strawberry.type
    class People:
        name: str
        age: int

    lorem = People(name="Alex", age=30)

    assert asdict(lorem) == {
        "name": "Alex",
        "age": 30,
    }


def test_convert_complex_type_to_dictionary():
    @strawberry.enum
    class Count(Enum):
        TWO = "two"
        FOUR = "four"

    @strawberry.type
    class Animal:
        legs: Count

    @strawberry.type
    class People:
        name: str
        animals: list[Animal]

    lorem = People(
        name="Kevin", animals=[Animal(legs=Count.TWO), Animal(legs=Count.FOUR)]
    )

    assert asdict(lorem) == {
        "name": "Kevin",
        "animals": [
            {"legs": Count.TWO},
            {"legs": Count.FOUR},
        ],
    }


def test_convert_input_to_dictionary():
    @strawberry.input
    class QnaInput:
        title: str
        description: str
        tags: list[str] | None = strawberry.field(default=None)

    title = "Where is the capital of United Kingdom?"
    description = "London is the capital of United Kingdom."
    qna = QnaInput(title=title, description=description)

    assert asdict(qna) == {
        "title": title,
        "description": description,
        "tags": None,
    }



import textwrap
from textwrap import dedent
from unittest.mock import patch

import pytest
from graphql import GraphQLError, ValidationRule, validate

import strawberry
from strawberry.extensions import AddValidationRules, DisableValidation


@pytest.mark.parametrize("validate_queries", [True, False])
@patch("strawberry.schema.schema.validate", wraps=validate)
def test_enabling_query_validation_sync(mock_validate, validate_queries):
    @strawberry.type
    class Query:
        example: str | None = None

    extensions = []
    if validate_queries is False:
        extensions.append(DisableValidation())

    schema = strawberry.Schema(
        query=Query,
        extensions=extensions,
    )

    query = """
        query {
            example
        }
    """

    result = schema.execute_sync(
        query,
        root_value=Query(),
    )

    assert not result.errors

    assert mock_validate.called is validate_queries


@pytest.mark.asyncio
@pytest.mark.parametrize("validate_queries", [True, False])
async def test_enabling_query_validation(validate_queries):
    @strawberry.type
    class Query:
        example: str | None = None

    extensions = []
    if validate_queries is False:
        extensions.append(DisableValidation())

    schema = strawberry.Schema(
        query=Query,
        extensions=extensions,
    )

    query = """
        query {
            example
        }
    """

    with patch("strawberry.schema.schema.validate", wraps=validate) as mock_validate:
        result = await schema.execute(
            query,
            root_value=Query(),
        )

        assert not result.errors

        assert mock_validate.called is validate_queries


@pytest.mark.asyncio
async def test_invalid_query_with_validation_enabled():
    @strawberry.type
    class Query:
        example: str | None = None

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
    """

    result = await schema.execute(query, root_value=Query())

    assert str(result.errors[0]) == (
        "Syntax Error: Expected Name, found <EOF>.\n\n"
        "GraphQL request:4:5\n"
        "3 |             example\n"
        "4 |     \n"
        "  |     ^"
    )


@pytest.mark.asyncio
async def test_asking_for_wrong_field():
    @strawberry.type
    class Query:
        example: str | None = None

    schema = strawberry.Schema(query=Query, extensions=[DisableValidation()])

    query = """
        query {
            sample
        }
    """

    result = await schema.execute(
        query,
        root_value=Query(),
    )

    assert result.errors is None
    assert result.data == {}


@pytest.mark.asyncio
async def test_sending_wrong_variables():
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self, value: str) -> int:
            return 1

    schema = strawberry.Schema(query=Query, extensions=[DisableValidation()])

    query = """
        query {
            example(value: 123)
        }
    """

    result = await schema.execute(
        query,
        root_value=Query(),
    )

    assert (
        str(result.errors[0])
        == textwrap.dedent(
            """
            Argument 'value' has invalid value 123.

            GraphQL request:3:28
            2 |         query {
            3 |             example(value: 123)
              |                            ^
            4 |         }
            """
        ).strip()
    )


@pytest.mark.asyncio
async def test_logging_exceptions(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> int:
            raise ValueError("test")

    schema = strawberry.Schema(query=Query)

    query = dedent(
        """
        query {
            example
        }
    """
    ).strip()

    result = await schema.execute(
        query,
        root_value=Query(),
    )

    assert result.errors
    assert len(result.errors) == 1

    # Exception was logged
    assert len(caplog.records) == 1
    record = caplog.records[0]

    assert record.levelname == "ERROR"
    assert (
        record.message
        == dedent(
            """
        test

        GraphQL request:2:5
        1 | query {
        2 |     example
          |     ^
        3 | }
    """
        ).strip()
    )
    assert record.name == "strawberry.execution"
    assert record.exc_info[0] is ValueError


@pytest.mark.asyncio
async def test_logging_graphql_exceptions(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> int:
            return None  # type: ignore

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
        }
    """

    result = await schema.execute(
        query,
        root_value=Query(),
    )

    assert len(result.errors) == 1

    # Exception was logged
    assert len(caplog.records) == 1
    record = caplog.records[0]

    assert record.levelname == "ERROR"
    assert record.name == "strawberry.execution"
    assert record.exc_info[0] is TypeError


@pytest.mark.asyncio
async def test_logging_parsing_error(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> str:
            return "hi"

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
    """

    result = await schema.execute(
        query,
        root_value=Query(),
    )

    assert result.errors
    assert len(result.errors) == 1

    # Exception was logged
    assert len(caplog.records) == 1
    record = caplog.records[0]

    assert record.levelname == "ERROR"
    assert record.name == "strawberry.execution"
    assert "Syntax Error" in record.message


def test_logging_parsing_error_sync(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> str:
            return "hi"

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
    """

    result = schema.execute_sync(
        query,
        root_value=Query(),
    )

    assert result.errors
    assert len(result.errors) == 1

    # Exception was logged
    assert len(caplog.records) == 1
    record = caplog.records[0]

    assert record.levelname == "ERROR"
    assert record.name == "strawberry.execution"
    assert "Syntax Error" in record.message


@pytest.mark.asyncio
async def test_logging_validation_errors(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> str:
            return "hi"

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example {
                foo
            }
            missingField
        }
    """

    result = await schema.execute(
        query,
        root_value=Query(),
    )

    assert result.errors
    assert len(result.errors) == 2

    # Exception was logged
    assert len(caplog.records) == 2
    record1 = caplog.records[0]
    assert record1.levelname == "ERROR"
    assert record1.name == "strawberry.execution"
    assert "Field 'example' must not have a selection" in record1.message

    record2 = caplog.records[1]
    assert record2.levelname == "ERROR"
    assert record2.name == "strawberry.execution"
    assert "Cannot query field 'missingField'" in record2.message


def test_logging_validation_errors_sync(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> str:
            return "hi"

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example {
                foo
            }
            missingField
        }
    """

    result = schema.execute_sync(
        query,
        root_value=Query(),
    )

    assert result.errors
    assert len(result.errors) == 2

    # Exception was logged
    assert len(caplog.records) == 2
    record1 = caplog.records[0]
    assert record1.levelname == "ERROR"
    assert record1.name == "strawberry.execution"
    assert "Field 'example' must not have a selection" in record1.message

    record2 = caplog.records[1]
    assert record2.levelname == "ERROR"
    assert record2.name == "strawberry.execution"
    assert "Cannot query field 'missingField'" in record2.message


def test_overriding_process_errors(caplog: pytest.LogCaptureFixture):
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> int:
            return None  # type: ignore

    execution_errors = []

    class CustomSchema(strawberry.Schema):
        def process_errors(self, errors, execution_context):
            nonlocal execution_errors
            execution_errors = errors

    schema = CustomSchema(query=Query)

    query = """
        query {
            example
        }
    """

    result = schema.execute_sync(
        query,
        root_value=Query(),
    )

    assert len(result.errors) == 1
    assert len(execution_errors) == 1
    assert result.errors == execution_errors

    # Exception wasn't logged
    assert caplog.records == []


def test_adding_custom_validation_rules():
    @strawberry.type
    class Query:
        example: str | None = None
        another_example: str | None = None

    class CustomRule(ValidationRule):
        def enter_field(self, node, *args: str) -> None:
            if node.name.value == "example":
                self.report_error(GraphQLError("Can't query field 'example'"))

    schema = strawberry.Schema(
        query=Query,
        extensions=[
            AddValidationRules([CustomRule]),
        ],
    )

    result = schema.execute_sync(
        "{ example }",
        root_value=Query(),
    )

    assert str(result.errors[0]) == "Can't query field 'example'"

    result = schema.execute_sync(
        "{ anotherExample }",
        root_value=Query(),
    )
    assert not result.errors


def test_partial_responses():
    @strawberry.type
    class Query:
        @strawberry.field
        def example(self) -> str:
            return "hi"

        @strawberry.field
        def this_fails(self) -> str | None:
            raise ValueError("this field fails")

    schema = strawberry.Schema(query=Query)

    query = """
        query {
            example
            thisFails
        }
    """

    result = schema.execute_sync(query)

    assert result.data == {"example": "hi", "thisFails": None}
    assert result.errors
    assert result.errors[0].message == "this field fails"



import datetime
from typing import Annotated, Generic, TypeVar

import pytest

import strawberry
from strawberry.types.base import (
    StrawberryList,
    StrawberryOptional,
    StrawberryTypeVar,
    get_object_definition,
)
from strawberry.types.union import StrawberryUnion

T = TypeVar("T")


def test_basic_generic():
    directive = object()

    @strawberry.type
    class Edge(Generic[T]):
        node_field: T = strawberry.field(directives=[directive])

    definition = get_object_definition(Edge, strict=True)
    assert definition.is_graphql_generic
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "node_field"
    assert isinstance(field.type, StrawberryTypeVar)
    assert field.type.type_var is T

    # let's make a copy of this generic type
    copy = get_object_definition(Edge, strict=True).copy_with({"T": str})

    definition_copy = get_object_definition(copy, strict=True)

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "node_field"
    assert field_copy.type is str
    assert field_copy.directives == [directive]


def test_generics_nested():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Connection(Generic[T]):
        edge: Edge[T]

    definition = get_object_definition(Connection, strict=True)
    assert definition.is_graphql_generic
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "edge"
    assert get_object_definition(field.type, strict=True).type_params == [T]

    # let's make a copy of this generic type
    definition_copy = get_object_definition(
        get_object_definition(Connection, strict=True).copy_with({"T": str}),
        strict=True,
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "edge"


def test_generics_name():
    @strawberry.type(name="AnotherName")
    class EdgeName:
        node: str

    @strawberry.type
    class Connection(Generic[T]):
        edge: T

    definition_copy = get_object_definition(
        get_object_definition(Connection, strict=True).copy_with({"T": EdgeName}),
        strict=True,
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "edge"


def test_generics_nested_in_list():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Connection(Generic[T]):
        edges: list[Edge[T]]

    definition = get_object_definition(Connection, strict=True)
    assert definition.is_graphql_generic
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "edges"
    assert isinstance(field.type, StrawberryList)
    assert get_object_definition(field.type.of_type, strict=True).type_params == [T]

    # let's make a copy of this generic type
    definition_copy = get_object_definition(
        definition.copy_with({"T": str}),
        strict=True,
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "edges"
    assert isinstance(field_copy.type, StrawberryList)


def test_list_inside_generic():
    T = TypeVar("T")

    @strawberry.type
    class Value(Generic[T]):
        valuation_date: datetime.date
        value: T

    @strawberry.type
    class Foo:
        string: Value[str]
        strings: Value[list[str]]
        optional_string: Value[str | None]
        optional_strings: Value[list[str] | None]

    definition = get_object_definition(Foo, strict=True)
    assert not definition.is_graphql_generic

    [
        string_field,
        strings_field,
        optional_string_field,
        optional_strings_field,
    ] = definition.fields
    assert string_field.python_name == "string"
    assert strings_field.python_name == "strings"
    assert optional_string_field.python_name == "optional_string"
    assert optional_strings_field.python_name == "optional_strings"


def test_generic_with_optional():
    @strawberry.type
    class Edge(Generic[T]):
        node: T | None

    definition = get_object_definition(Edge, strict=True)
    assert definition.is_graphql_generic
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "node"
    assert isinstance(field.type, StrawberryOptional)
    assert isinstance(field.type.of_type, StrawberryTypeVar)
    assert field.type.of_type.type_var is T

    # let's make a copy of this generic type
    definition_copy = get_object_definition(
        definition.copy_with({"T": str}), strict=True
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "node"
    assert isinstance(field_copy.type, StrawberryOptional)
    assert field_copy.type.of_type is str


def test_generic_with_list():
    @strawberry.type
    class Connection(Generic[T]):
        edges: list[T]

    definition = get_object_definition(Connection, strict=True)
    assert definition.is_graphql_generic
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "edges"
    assert isinstance(field.type, StrawberryList)
    assert isinstance(field.type.of_type, StrawberryTypeVar)
    assert field.type.of_type.type_var is T

    # let's make a copy of this generic type
    definition_copy = get_object_definition(
        definition.copy_with({"T": str}), strict=True
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "edges"
    assert isinstance(field_copy.type, StrawberryList)
    assert field_copy.type.of_type is str


def test_generic_with_list_of_optionals():
    @strawberry.type
    class Connection(Generic[T]):
        edges: list[T | None]

    definition = get_object_definition(Connection, strict=True)
    assert definition.is_graphql_generic
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "edges"
    assert isinstance(field.type, StrawberryList)
    assert isinstance(field.type.of_type, StrawberryOptional)
    assert isinstance(field.type.of_type.of_type, StrawberryTypeVar)
    assert field.type.of_type.of_type.type_var is T

    # let's make a copy of this generic type
    definition_copy = get_object_definition(
        definition.copy_with({"T": str}), strict=True
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "edges"
    assert isinstance(field_copy.type, StrawberryList)
    assert isinstance(field_copy.type.of_type, StrawberryOptional)
    assert field_copy.type.of_type.of_type is str


def test_generics_with_unions():
    @strawberry.type
    class Error:
        message: str

    @strawberry.type
    class Edge(Generic[T]):
        node: Error | T

    definition = get_object_definition(Edge, strict=True)
    assert definition.type_params == [T]

    [field] = definition.fields
    assert field.python_name == "node"
    assert isinstance(field.type, StrawberryUnion)
    assert field.type.types == (Error, T)

    # let's make a copy of this generic type
    @strawberry.type
    class Node:
        name: str

    definition_copy = get_object_definition(
        definition.copy_with({"T": Node}), strict=True
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []

    [field_copy] = definition_copy.fields
    assert field_copy.python_name == "node"
    assert isinstance(field_copy.type, StrawberryUnion)
    assert field_copy.type.types == (Error, Node)


def test_using_generics():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class User:
        name: str

    @strawberry.type
    class Query:
        user: Edge[User]

    definition = get_object_definition(Query, strict=True)

    [field] = definition.fields
    assert field.python_name == "user"

    user_edge_definition = get_object_definition(field.type, strict=True)
    assert not user_edge_definition.is_graphql_generic

    [node_field] = user_edge_definition.fields
    assert node_field.python_name == "node"
    assert node_field.type is User


def test_using_generics_nested():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Connection(Generic[T]):
        edges: Edge[T]

    @strawberry.type
    class User:
        name: str

    @strawberry.type
    class Query:
        users: Connection[User]

    connection_definition = get_object_definition(Connection, strict=True)
    assert connection_definition.is_graphql_generic
    assert connection_definition.type_params == [T]

    query_definition = get_object_definition(Query, strict=True)

    [user_field] = query_definition.fields
    assert user_field.python_name == "users"

    user_connection_definition = get_object_definition(user_field.type, strict=True)
    assert not user_connection_definition.is_graphql_generic

    [edges_field] = user_connection_definition.fields
    assert edges_field.python_name == "edges"


def test_using_generics_raises_when_missing_annotation():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class User:
        name: str

    error_message = (
        f'Query fields cannot be resolved. The type "{Edge!r}" '
        "is generic, but no type has been passed"
    )

    @strawberry.type
    class Query:
        user: Edge

    with pytest.raises(TypeError, match=error_message):
        strawberry.Schema(Query)


def test_using_generics_raises_when_missing_annotation_nested():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Connection(Generic[T]):
        edges: list[Edge[T]]

    @strawberry.type
    class User:
        name: str

    error_message = (
        f'Query fields cannot be resolved. The type "{Connection!r}" '
        "is generic, but no type has been passed"
    )

    @strawberry.type
    class Query:
        users: Connection

    with pytest.raises(TypeError, match=error_message):
        strawberry.Schema(Query)


def test_generics_inside_optional():
    @strawberry.type
    class Error:
        message: str

    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Query:
        user: Edge[str] | None

    query_definition = get_object_definition(Query, strict=True)
    assert query_definition.type_params == []

    [field] = query_definition.fields
    assert field.python_name == "user"
    assert isinstance(field.type, StrawberryOptional)

    str_edge_definition = get_object_definition(field.type.of_type, strict=True)
    assert not str_edge_definition.is_graphql_generic


def test_generics_inside_list():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Query:
        user: list[Edge[str]]

    query_definition = get_object_definition(Query, strict=True)
    assert query_definition.type_params == []

    [field] = query_definition.fields
    assert field.python_name == "user"
    assert isinstance(field.type, StrawberryList)

    str_edge_definition = get_object_definition(field.type.of_type, strict=True)
    assert not str_edge_definition.is_graphql_generic


def test_generics_inside_unions():
    @strawberry.type
    class Error:
        message: str

    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Query:
        user: Edge[str] | Error

    query_definition = get_object_definition(Query, strict=True)
    assert query_definition.type_params == []

    [field] = query_definition.fields
    assert field.python_name == "user"
    assert not isinstance(field.type, StrawberryOptional)

    union = field.type
    assert isinstance(union, StrawberryUnion)
    assert not get_object_definition(union.types[0], strict=True).is_graphql_generic


def test_multiple_generics_inside_unions():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.type
    class Query:
        user: Edge[int] | Edge[str]

    query_definition = get_object_definition(Query, strict=True)
    assert query_definition.type_params == []

    [user_field] = query_definition.fields
    assert user_field.python_name == "user"
    assert not isinstance(user_field.type, StrawberryOptional)

    union = user_field.type
    assert isinstance(union, StrawberryUnion)

    int_edge_definition = get_object_definition(union.types[0], strict=True)
    assert not int_edge_definition.is_graphql_generic
    assert int_edge_definition.fields[0].type is int

    str_edge_definition = get_object_definition(union.types[1], strict=True)
    assert not str_edge_definition.is_graphql_generic
    assert str_edge_definition.fields[0].type is str


def test_union_inside_generics():
    @strawberry.type
    class Dog:
        name: str

    @strawberry.type
    class Cat:
        name: str

    @strawberry.type
    class Connection(Generic[T]):
        nodes: list[T]

    DogCat = Annotated[Dog | Cat, strawberry.union("DogCat")]

    @strawberry.type
    class Query:
        connection: Connection[DogCat]

    query_definition = get_object_definition(Query, strict=True)
    assert query_definition.type_params == []

    [connection_field] = query_definition.fields
    assert connection_field.python_name == "connection"
    assert not isinstance(connection_field, StrawberryOptional)

    dog_cat_connection_definition = get_object_definition(
        connection_field.type, strict=True
    )

    [node_field] = dog_cat_connection_definition.fields
    assert isinstance(node_field.type, StrawberryList)

    union = dog_cat_connection_definition.fields[0].type.of_type
    assert isinstance(union, StrawberryUnion)


def test_anonymous_union_inside_generics():
    @strawberry.type
    class Dog:
        name: str

    @strawberry.type
    class Cat:
        name: str

    @strawberry.type
    class Connection(Generic[T]):
        nodes: list[T]

    @strawberry.type
    class Query:
        connection: Connection[Dog | Cat]

    definition = get_object_definition(Query, strict=True)
    assert definition.type_params == []

    [connection_field] = definition.fields
    assert connection_field.python_name == "connection"

    dog_cat_connection_definition = get_object_definition(
        connection_field.type, strict=True
    )

    [node_field] = dog_cat_connection_definition.fields
    assert isinstance(node_field.type, StrawberryList)

    union = node_field.type.of_type
    assert isinstance(union, StrawberryUnion)


def test_using_generics_with_interfaces():
    @strawberry.type
    class Edge(Generic[T]):
        node: T

    @strawberry.interface
    class WithName:
        name: str

    @strawberry.type
    class Query:
        user: Edge[WithName]

    query_definition = get_object_definition(Query, strict=True)

    [user_field] = query_definition.fields
    assert user_field.python_name == "user"

    with_name_definition = get_object_definition(user_field.type, strict=True)
    assert not with_name_definition.is_graphql_generic

    [node_field] = with_name_definition.fields
    assert node_field.python_name == "node"
    assert node_field.type is WithName


def test_generic_with_arguments():
    T = TypeVar("T")

    @strawberry.type
    class Collection(Generic[T]):
        @strawberry.field
        def by_id(self, ids: list[int]) -> list[T]:
            return []

    @strawberry.type
    class Post:
        name: str

    @strawberry.type
    class Query:
        user: Collection[Post]

    query_definition = get_object_definition(Query, strict=True)

    [user_field] = query_definition.fields
    assert user_field.python_name == "user"

    post_collection_definition = get_object_definition(user_field.type, strict=True)
    assert not post_collection_definition.is_graphql_generic

    [by_id_field] = post_collection_definition.fields
    assert by_id_field.python_name == "by_id"
    assert isinstance(by_id_field.type, StrawberryList)
    assert by_id_field.type.of_type is Post

    [ids_argument] = by_id_field.arguments
    assert ids_argument.python_name == "ids"
    assert isinstance(ids_argument.type, StrawberryList)
    assert ids_argument.type.of_type is int


def test_federation():
    @strawberry.federation.type(keys=["id"])
    class Edge(Generic[T]):
        id: strawberry.ID
        node_field: T

    definition = get_object_definition(Edge, strict=True)

    definition_copy = get_object_definition(
        definition.copy_with({"T": str}), strict=True
    )

    assert not definition_copy.is_graphql_generic
    assert definition_copy.type_params == []
    assert definition_copy.directives == definition.directives

    [field1_copy, field2_copy] = definition_copy.fields

    assert field1_copy.python_name == "id"
    assert field1_copy.type is strawberry.ID

    assert field2_copy.python_name == "node_field"
    assert field2_copy.type is str



# - 1. test fragments
# - 2. test variables
# - 3. test input objects
# - 4. test mutations (raise?)
# - 5. test subscriptions (raise)

from pathlib import Path

import pytest
from pytest_snapshot.plugin import Snapshot

from strawberry.codegen import QueryCodegen, QueryCodegenPlugin
from strawberry.codegen.exceptions import (
    MultipleOperationsProvidedError,
    NoOperationNameProvidedError,
    NoOperationProvidedError,
)
from strawberry.codegen.plugins.python import PythonPlugin
from strawberry.codegen.plugins.typescript import TypeScriptPlugin

HERE = Path(__file__).parent
QUERIES = list(HERE.glob("queries/*.graphql"))


@pytest.mark.parametrize(
    ("plugin_class", "plugin_name", "extension"),
    [
        (PythonPlugin, "python", "py"),
        (TypeScriptPlugin, "typescript", "ts"),
    ],
    ids=["python", "typescript"],
)
@pytest.mark.parametrize("query", QUERIES, ids=[x.name for x in QUERIES])
def test_codegen(
    query: Path,
    plugin_class: type[QueryCodegenPlugin],
    plugin_name: str,
    extension: str,
    snapshot: Snapshot,
    schema,
):
    generator = QueryCodegen(schema, plugins=[plugin_class(query)])

    result = generator.run(query.read_text())

    code = result.to_string()

    snapshot.snapshot_dir = HERE / "snapshots" / plugin_name
    snapshot.assert_match(code, f"{query.with_suffix('').stem}.{extension}")


def test_codegen_fails_if_no_operation_name(schema, tmp_path):
    query = tmp_path / "query.graphql"
    data = "query { hello }"
    with query.open("w") as f:
        f.write(data)

    generator = QueryCodegen(schema, plugins=[PythonPlugin(query)])

    with pytest.raises(NoOperationNameProvidedError):
        generator.run(data)


def test_codegen_fails_if_no_operation(schema, tmp_path):
    query = tmp_path / "query.graphql"
    data = "type X { hello: String }"
    with query.open("w") as f:
        f.write(data)

    generator = QueryCodegen(schema, plugins=[PythonPlugin(query)])

    with pytest.raises(NoOperationProvidedError):
        generator.run(data)


def test_fails_with_multiple_operations(schema, tmp_path):
    query = tmp_path / "query.graphql"
    data = "query { hello } query { world }"
    with query.open("w") as f:
        f.write(data)

    generator = QueryCodegen(schema, plugins=[PythonPlugin(query)])

    with pytest.raises(MultipleOperationsProvidedError):
        generator.run(data)



import importlib
from collections.abc import Generator
from typing import Any

import pytest

from tests.views.schema import schema

from .clients.base import HttpClient


def _get_http_client_classes() -> Generator[Any, None, None]:
    for client, module, marks in [
        ("AioHttpClient", "aiohttp", [pytest.mark.aiohttp]),
        ("AsgiHttpClient", "asgi", [pytest.mark.asgi]),
        ("AsyncDjangoHttpClient", "async_django", [pytest.mark.django]),
        ("AsyncFlaskHttpClient", "async_flask", [pytest.mark.flask]),
        ("ChannelsHttpClient", "channels", [pytest.mark.channels]),
        ("ChaliceHttpClient", "chalice", [pytest.mark.chalice]),
        ("DjangoHttpClient", "django", [pytest.mark.django]),
        ("FastAPIHttpClient", "fastapi", [pytest.mark.fastapi]),
        ("FlaskHttpClient", "flask", [pytest.mark.flask]),
        ("QuartHttpClient", "quart", [pytest.mark.quart]),
        ("SanicHttpClient", "sanic", [pytest.mark.sanic]),
        ("LitestarHttpClient", "litestar", [pytest.mark.litestar]),
        (
            "SyncChannelsHttpClient",
            "channels",
            [pytest.mark.channels, pytest.mark.django_db],
        ),
    ]:
        try:
            client_class = getattr(
                importlib.import_module(f".{module}", package="tests.http.clients"),
                client,
            )
        except ImportError:
            client_class = None

        yield pytest.param(
            client_class,
            marks=[
                *marks,
                pytest.mark.skipif(
                    client_class is None, reason=f"Client {client} not found"
                ),
            ],
        )


@pytest.fixture(params=_get_http_client_classes())
def http_client_class(request: Any) -> type[HttpClient]:
    return request.param


@pytest.fixture
def http_client(http_client_class: type[HttpClient]) -> HttpClient:
    return http_client_class(schema)
